% Copyright 2008 INRIA, F. Fages & J. Martin
% License GPL
%
% file icg.pl
%

:- ensure_loaded(library(terms)).
:- ensure_loaded(library(lists)).
:- ensure_loaded([parser]).
:- ensure_loaded([subst]).
:- ensure_loaded([order]).
:- ensure_loaded([eval]).
:- ensure_loaded([cse]).

:- dynamic record_uid_table/2,
	   rewrite_term_hash/3,
	   search_mode/2,
	   statement/3,
	   statement_search/3,
	   rcp_heuristics/2,
	   rcp_statement/3,
	   conjorder_terms/1,
	   disjorder_terms/1.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% intermediate code generation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gen_intcode(IntermediateCode) :-
	query(Query),
	!,
	init_intcode_gen(CStack, Env),
	write('\"-------------------- Query\"'),nl,
				%write(Query),nl,
	rewrite_term(Query, CStack, Env, IntCode),
	normalize_term(IntCode, NormalizedCode),
	cse_commutativity(NormalizedCode, NormCommCode),
	cse(NormCommCode, _CSECode),
	IntermediateCode = NormalizedCode,
	write('\"-------------------- Statements\"'),nl,
	rewrite_statements. 
gen_intcode(1).

init_intcode_gen(CStack, Env) :-
	init_record_uid_table,
	counter_init(local_vars),
	counter_init(global_vars),
	counter_init(var_order_criteria),
	counter_init(val_order_criteria),
	counter_init(it_rank),
	counter_init(search_statement),	
	init_rewrite_statements,
	init_rewrite_statements_search,
	retractall(rewrite_term_hash(_, _, _)),
	retractall(code_gen_mode(_)),
	set_cg_mode(static),
	retractall(search_mode(_, _)),
	assert(search_mode(false, none)),
	CStack = [and, solve],
	env_new(Env).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generic rewriting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% free var
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term(Term, _CStack, _Env, Term) :-
	is_free_var(Term),
	!.

% var
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term(Term, _CStack, _Env, Term) :-
	is_var(Term),
	!.

% bool
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term(true, _CStack, _Env, 1) :-
	!.

rewrite_term(false, _CStack, _Env, 0) :-
	!.

% int
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term(Term, _CStack, _Env, Term) :-
	integer(Term),
	!.

% special atoms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term(^, _, _, ^) :-
	!.

% term rewriting memoization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term(Term, CStack, Env, Rewritten) :-
	srch_mode(_),
	!,
	rewrite_term_rec(Term, CStack, Env, Rewritten).
rewrite_term(Term, CStack, Env, Rewritten) :-
	cg_mode(dynamic),
	!,
	rewrite_term_rec(Term, CStack, Env, Rewritten).
rewrite_term(Term, CStack, Env, Rewritten) :-
	!,
	term_hash(Term, Hash),
	(	  
	  rewrite_term_hash(Hash, Term, Rewritten)
	->
	  true
	;
	  rewrite_term_rec(Term, CStack, Env, Rewritten),
	  assertz(rewrite_term_hash(Hash, Term, Rewritten))
	).

% name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(zot(min_integer), _, _, min_integer) :-
	!.
rewrite_term_rec(zot(max_integer), _, _, max_integer) :-
	!.

% % function
% %%%%%%%%%%%%%%%%%%%%%
% rewrite_term_rec(zot(Term), CStack, Env, RTerm) :-
% 	constant_function(Term, Head, Body),
% 	!,
% 	rewrite_term(constant_function_term(Term, Head, Body), CStack, Env, RTerm).

% rewrite_term_rec(constant_function_term(Term, _Head, _Body), _CStack, _Env, Term) :-
% 	cg_mode(dynamic),
% 	!.
% rewrite_term_rec(constant_function_term(Term, Head, Body), CStack, Env, RTerm) :- 
% 	!,
% 	counter_init(local_vars),
% 	env_new(EmptyEnv),	
% 	substitution(EmptyEnv, Body, Term, _, BodySubst),
% 	rewrite_term(BodySubst, [Head | CStack], Env, RTerm).

% relation
%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(zot(Term), CStack, Env, RTerm) :-
	constant_relation(Term, Head, Body),
	!,        
	rewrite_term(constant_relation_term(Term, Head, Body), CStack, Env, RTerm).

rewrite_term_rec(constant_relation_term(Term, _Head, _Body), _CStack, _Env, Term) :-
	cg_mode(dynamic),
	!.
rewrite_term_rec(constant_relation_term(Term, Head, Body), CStack, Env, RTerm) :-
	!,
	env_new(EmptyEnv),
        counter_init(local_vars),
	substitution(EmptyEnv, Body, Term, _, BodySubst),
	rewrite_term(BodySubst, [Head | CStack], Env, RT),
        (
           CStack = [Op | CSubStack]
        ->
           skip_logops(CSubStack, SCStack, SrcTerm),
           insert_label_node(Term, [Op, SrcTerm | SCStack], RT, RTerm)       
        ;  
           RTerm = RT
        ).

% geost ecstr param. 
%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(zot(Term), [geost_constraint | _CStack], _Env,  Term) :-
	memberchk(Term, [non_overlapping, included, visible]),
	!.
	
% not defined 
%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(zot(Term), CStack, _Env,  _) :-
	!,
	raise_exception(generic_rewriting(not_defined, [Term], CStack)).

% ordered pair (Key = Val) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(K = V, CStack, Env, K = RV) :-
	is_ident(K),
	!,
	rewrite_term(V, CStack, Env, RV).

% expr (arithop) expr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(Term, CStack, Env, RTerm) :-
	is_arithop(Term),
	!,
	Term =.. [Op, ExprL, ExprR],
	rewrite_term(ExprL, CStack, Env, EExprL),
	rewrite_term(ExprR, CStack, Env, EExprR),
	RT =.. [Op, EExprL, EExprR],
	eval(RT, RTerm).

% arithop(expr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(Term, CStack, Env, RTerm) :-
	is_arithop(Term),
	!,
	Term =.. [Op, Expr],
	rewrite_term(Expr, CStack, Env, EExpr),
	RT =.. [Op, EExpr],
	eval(RT, RTerm).

% expr (compop) expr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(Term, CStack, Env, RTerm) :-
	is_compop(Term),
	!,
	Term =.. [CompOp, ExprL, ExprR],
	rewrite_term(ExprL, CStack, Env, EExprL),
	rewrite_term(ExprR, CStack, Env, EExprR),
	RT =.. [CompOp, EExprL, EExprR],
	eval(RT, RTerm).

% expr in expr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(VExpr in Expr, CStack, Env, V in E) :-
	cg_mode(dynamic),
	!,
	rewrite_term(VExpr, CStack, Env, V),
	rewrite_term(Expr, CStack, Env, E). 
rewrite_term_rec(V in E, CStack, Env, X in L) :-
	!,
	rewrite_term(E, [in | CStack], Env, L),
	reduced_to_type_or_excpt(list, E, L, CStack),
	rewrite_term(V, CStack, Env, X),
	reduced_to_type_or_excpt(variable, V, X, CStack). 

% domain
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(domain, [VExpr, MinExpr, MaxExpr]),
		 CStack, Env, domain(variables(Expr), Min, Max)) :-
	cg_mode(dynamic),
	!,
	rewrite_list([VExpr, MinExpr, MaxExpr], CStack, Env, [Expr, Min, Max]).
rewrite_term_rec(fot(domain, [VExpr, MinExpr, MaxExpr]),
		 [Op | CStack], Env, RTerm) :-
	!,
	rewrite_list([VExpr, MinExpr, MaxExpr], CStack, Env, [V, Min, Max]),
	variables(V, Vars),
	skip_logops(CStack, SCStack, SrcTerm),
	insert_label_node(SrcTerm, [Op, SrcTerm | SCStack],
			  domain(Vars, Min, Max), RTerm).

% domain lower bound
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(domain_min, [Expr]), CStack, Env, domain_min(V)) :-
	cg_mode(dynamic),
	!,	
	rewrite_term(Expr, CStack, Env, V).
rewrite_term_rec(fot(domain_min, [Expr]), CStack, Env, domain_min(V)) :-
	!,	
	rewrite_term(Expr, CStack, Env, V),
	reduced_to_type_or_excpt(variable, Expr, V, CStack).

% domain upper bound
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(domain_max, [Expr]), CStack, Env, domain_max(V)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(Expr, CStack, Env, V).
rewrite_term_rec(fot(domain_max, [Expr]), CStack, Env, domain_max(V)) :-
	!,
	rewrite_term(Expr, CStack, Env, V),
	reduced_to_type_or_excpt(variable, Expr, V, CStack).

% domain cardinality
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(domain_size, [Expr]), CStack, Env, domain_size(V)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(Expr, CStack, Env, V).
rewrite_term_rec(fot(domain_size, [Expr]), CStack, Env, domain_size(V)) :-
	!,
	rewrite_term(Expr, CStack, Env, V),
	reduced_to_type_or_excpt(variable, Expr, V, CStack).

% global constraint all different
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(all_different, [LExpr]),
		 CStack, Env, all_different(variables(Expr))) :-
	cg_mode(dynamic),
	!,
	rewrite_term(LExpr, CStack, Env, Expr).
rewrite_term_rec(fot(all_different, [LExpr]),
		 CStack, Env, all_different(Vars)) :-
	!,
	rewrite_term(LExpr, CStack, Env, L),
	variables(L, Vars).

% global constraint lexicographic chain
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(lexicographic, [LExpr]),
		 CStack, Env, lexicographic(Expr)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(LExpr, CStack, Env, Expr).
rewrite_term_rec(fot(lexicographic, [LExpr]),
		 CStack, Env, lexicographic(L)) :-
	!,
	rewrite_term(LExpr, CStack, Env, L),
	reduced_to_type_or_excpt(list_of_list_of_var, LExpr, L, CStack).

rewrite_term_rec(fot(lexicographic_strict, [LExpr]),
		 CStack, Env, lexicographic_strict(Expr)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(LExpr, CStack, Env, Expr).
rewrite_term_rec(fot(lexicographic_strict, [LExpr]),
		 CStack, Env, lexicographic_strict(L)) :-
	!,
	rewrite_term(LExpr, CStack, Env, L),
	reduced_to_type_or_excpt(list_of_list_of_var, LExpr, L, CStack).

% variables 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(variables, [Term]), CStack, Env, RTerm) :-
	!,
	rewrite_term(Term, CStack, Env, RT),
	variables(RT, RTerm). 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% search / labeling / optimization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% search dynamic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(search, [Term]), CStack, Env, search(RTerm)) :-
	cg_mode(dynamic),
				% conjorder_terms([]),
				% disjorder_terms([]),
	!,
	set_srch_mode(reordering),        
	rewrite_term(Term, [search | CStack], Env, RTerm),
	unset_srch_mode.
% rewrite_term_rec(fot(search, [Term]), CStack, Env, search_dyn(RTerm)) :-
% 	cg_mode(dynamic),
% 	!,
% 	set_srch_mode(reordering),
% 	init_cost_lists(CostLists),	
% 	rewrite_term_search(Term, and, CostLists, Env, CStack, RTerm),
% 	rewrite_statements_search,	
% 	unset_srch_mode.

% search static
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(search, [Term]), CStack, Env, search(RTerm)) :-
	conjorder_terms([]),
	disjorder_terms([]),
	!,
	set_srch_mode(direct),        
	rewrite_term(Term, [search | CStack], Env, RTerm),	
	unset_srch_mode.

rewrite_term_rec(fot(search, [Term]), CStack, Env, search(RTerm)) :-
	conjorder_terms(COCrits),
	disjorder_terms(DOCrits),
	!,
	set_srch_mode(reordering),
	rewrite_term(Term, [search | CStack], Env, RT),
	order_and_or_tree(RT, COCrits, DOCrits, RTerm),
	unset_srch_mode.

% expr labeling 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(labeling, [Term]), CStack, Env, labeling(variables(RTerm))) :-
	cg_mode(dynamic),
	!,
	rewrite_term(Term, [labeling | CStack], Env, RTerm).
rewrite_term_rec(fot(labeling, [Term]), [Op | CStack], Env, RTerm) :-
	!,
	rewrite_term(Term, [labeling | CStack], Env, PT),
	variables(PT, Vars),
	RT = labeling(Vars),
	skip_logops(CStack, SCStack, _SrcTerm),
	insert_label_node(none, [Op, none | SCStack], RT, RTerm).

% minimize 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(minimize, [Cost]), CStack, Env,
		 minimize(RCost)) :-
	!,
	rewrite_term(Cost, [minimize | CStack], Env, RCost).
rewrite_term_rec(fot(minimize, [Goal, Cost]), CStack, Env,
		 minimize(RGoal, RCost)) :-
	!,
	rewrite_term(Goal, [minimize | CStack], Env, RGoal),
	rewrite_term(Cost, [minimize | CStack], Env, RCost).

% maximize 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(maximize, [Cost]), CStack, Env,
		 minimize(RCost)) :-
	!,
	rewrite_term(Cost, [maximize | CStack], Env, RCost).
rewrite_term_rec(fot(maximize, [Goal, Cost]), CStack, Env,
		 maximize(RGoal, RCost)) :-
	!,
	rewrite_term(Goal, [maximize | CStack], Env, RGoal),	
	rewrite_term(Cost, [maximize | CStack], Env, RCost).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% heuristics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% variable ordering
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(variable_ordering, [Term]), CStack, Env,
		 variable_ordering(RTerm)) :-
	!,
	rewrite_term(Term, [variable_ordering | CStack], Env, RTerm).

% var.crit.order (least|greatest|any|is) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rewrite_term_rec(fot(Functor, [Expr]), [variable_ordering | CStack], Env, RTerm) :-
% 	icg_mode,
% 	lang(varordercrit_preds, Preds),
% 	memberchk(Functor/1, Preds),
% 	!,
% 	rewrite_term(Expr, [variable_ordering | CStack], Env, RExpr),	
% 	RTerm =.. [Functor, RExpr].
rewrite_term_rec(fot(Functor, [Expr]), [variable_ordering | CStack], Env, RTerm) :-
	cg_mode(dynamic),
	lang(varordercrit_preds, Preds),
	memberchk(Functor/1, Preds),
	!,
	variables_set(Expr, ExprVars),
	subst(^, fot(rcp_call, [var('Candidate', 0)]), Expr, SExpr),
	rewrite_term(SExpr, [variable_ordering | CStack], Env, RExpr),	
	counter_inc(var_order_criteria, C),
	append([C], [list(ExprVars)], HCallArgs),
	append([C], [var('Candidate', 0)], HeadArgs1),
	append(HeadArgs1, [list(ExprVars)], HeadArgs),	
	Head =.. [var_order_criterion | HeadArgs],
	HCall =.. [var_order_criterion | HCallArgs],
                                %	store_statement(Head, RExpr, function),
        store_statement(Head, RExpr, rule),	
	RTerm =.. [Functor, HCall].
rewrite_term_rec(fot(Functor, [Expr]), [variable_ordering | CStack], Env, RTerm) :-
	lang(varordercrit_preds, Preds),
	memberchk(Functor/1, Preds),
	!,
	rewrite_term(Expr, CStack, Env, RExpr),
	RTerm =.. [Functor, RExpr].

% value choice heuristics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(value_ordering, [Term]), CStack, Env,
		 value_ordering(RTerm)) :-
	!,
	rewrite_term(Term, [value_ordering | CStack], Env, RTerm).

% val.crit.order (up|down|step|bisect) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rewrite_term_rec(fot(Functor, [Expr]), [value_ordering | CStack], Env, RTerm) :-
% 	icg_mode,
% 	lang(valordercrit_preds, Preds),
% 	memberchk(Functor/1, Preds),
% 	!,
% 	rewrite_term(Expr, [variable_ordering | CStack], Env, RExpr),	
% 	RTerm =.. [Functor, RExpr].
rewrite_term_rec(fot(Functor, [Expr]), [value_ordering | CStack], Env, RTerm) :-
	cg_mode(dynamic),
	lang(valordercrit_preds, Preds),
	memberchk(Functor/1, Preds),
	!,
	variables_set(Expr, ExprVars),
	subst(^, fot(rcp_call, [var('Candidate', 0)]), Expr, SExpr),	
	rewrite_term(SExpr, [value_ordering | CStack], Env, RExpr),	
	counter_inc(val_order_criteria, C),
	append([C], [list(ExprVars)], HCallArgs),
	append([C], [var('Candidate', 0)], HeadArgs1),
	append(HeadArgs1, [list(ExprVars)], HeadArgs),	
	Head =.. [val_order_criterion | HeadArgs],
	HCall =.. [val_order_criterion | HCallArgs],
                                %store_statement(Head, RExpr, function),
	store_statement(Head, RExpr, rule),
	RTerm =.. [Functor, HCall].
rewrite_term_rec(fot(Functor, [Expr]), [value_ordering | CStack], Env, RTerm) :-
	lang(valordercrit_preds, Preds),
	memberchk(Functor/1, Preds),
	!,
	rewrite_term(Expr, [value_ordering | CStack], Env, RExpr),
	RTerm =.. [Functor, RExpr].

% conjunct choice heuristics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(conjunct_ordering, [_Term]), _CStack, _Env, 1) :-
	cg_mode(dynamic),
	!.
rewrite_term_rec(fot(conjunct_ordering, [Term]), CStack, Env, 1) :-
	!,	
	rewrite_term(Term, CStack, Env, RTerm),
	store_term(RTerm, conjorder_terms).

% disjunct choice heuristics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(disjunct_ordering, [_Term]), _CStack, _Env, 1) :-
	cg_mode(dynamic),
	!.
rewrite_term_rec(fot(disjunct_ordering, [Term]), CStack, Env, 1) :-
	!,
	rewrite_term(Term, CStack, Env, RTerm),
	store_term(RTerm, disjorder_terms).

% fol crit.order (least | greatest) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(Functor, [Expr]), CStack, Env, RTerm) :-
	lang(folordercrit_preds, Preds),
	memberchk(Functor/1, Preds),
	!,
	rewrite_term(Expr, CStack, Env, RExpr),
	RTerm =.. [Functor, RExpr].

% conditional
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec((Expr if TExpr), CStack, Env, RTerm) :-
	variables_set(Expr, EVars),
	variables_set(TExpr, TVars),
	length(EVars, L),
	sublist(TVars, EVars, _, L, _),
	!,
	rewrite_term(cond(Expr, TExpr), CStack, Env, RTerm).

rewrite_term_rec(cond(Expr, TExpr), CStack, Env, (RExpr if TExpr)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(Expr, CStack, Env, RExpr).
rewrite_term_rec(cond(Expr, TExpr), _CStack, _Env, (Expr if TExpr)) :-
	!.
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% dynamic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(dynamic, [Term]), CStack, Env, dynamic(RTerm)) :-
	!,
	cg_mode(CGModeBefore),
	set_cg_mode(dynamic),
	(
	  srch_mode(SrchModeBefore)
	->	  
	  unset_srch_mode,
	  RestoreSrchMode = true
	;
	  RestoreSrchMode = false
	),
	rewrite_term(Term, CStack, Env, RTerm),
	(
	  RestoreSrchMode == true
	->
	  set_srch_mode(SrchModeBefore)
	;
	  true
	),
	set_cg_mode(CGModeBefore).	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% clpgui 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% gui_name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(gui_name, [VarExpr, ListExpr]), _CStack, _Env,
		 gui_name(VarExpr, ListExpr)) :-
	cg_mode(dynamic),
	!.

rewrite_term_rec(fot(gui_name, [VarExpr, ListExpr]), CStack, Env,
		 gui_name(Var, String)) :-
	!,
	rewrite_term(VarExpr, CStack, Env, Var),
	parse_or_excpt(var_or_int, Var, Var),	
	rewrite_list(ListExpr, CStack, Env, String),
	reduced_to_type_or_excpt(list, ListExpr, String, CStack).

% gui_control
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(gui_control, [zot(button), Term]), CStack, Env,
		 gui_control(button, Name, RTerm)) :-
	!,
	Term = fot(Name, _),
	rewrite_term(Term, CStack, Env, RTerm).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% write (compilation trace)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% write 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(write, [Term]), CStack, Env, rcp_write(RTerm)) :-
	!,
	rewrite_term(Term, CStack, Env, RTerm),
	format("\"--- User Output ---\"\n ~w \n ", [Term]),
	write("\"~~>\n\""),
	format(" ~w \n\"--- End User Output ---\"\n", [RTerm]).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% list functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% list - length 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(length, [Term]), CStack, Env, length(List)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(Term, CStack, Env, List).
rewrite_term_rec(fot(length, [Term]), CStack, Env, RTerm) :-
	!,
	rewrite_term(Term, CStack, Env, List),
	reduced_to_type_or_excpt(list, Term, List, CStack),
	rewrite_list(List, CStack, Env, RList),
	length(RList, RTerm).

% list - nth 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(nth, [N, Term]), CStack, Env,  nth(I, List)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(N, CStack, Env, I),
	rewrite_term(Term, CStack, Env, List).
rewrite_term_rec(fot(nth, [N, Term]), CStack, Env, RTerm) :-
	!,	
	rewrite_term(N, CStack, Env, I),
	reduced_to_type_or_excpt(integer, N, I, CStack),	
	rewrite_term(Term, CStack, Env, List),
	reduced_to_type_or_excpt(list, Term, List, CStack),
	rewrite_list(List, CStack, Env, RList),
	nth1(I, RList, RTerm).

% list - pos
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(pos, [Elem, Term]), CStack, Env, pos(RElem, List)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(Elem, CStack, Env, RElem),
	rewrite_term(Term, CStack, Env, List).
rewrite_term_rec(fot(pos, [Elem, Term]), CStack, Env, RTerm) :-
	!,
	rewrite_term(Elem, CStack, Env, RElem),
	rewrite_term(Term, CStack, Env, List),
	reduced_to_type_or_excpt(list, Term, List, CStack),	
	rewrite_list(List, CStack, Env, RList),
	nth1(RTerm, RList, RElem).

% list - cons
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(cons, [H, T]), _CStack, _Env, cons(H, T)) :-
	cg_mode(dynamic),
	!.
rewrite_term_rec(fot(cons, [H, T]), CStack, Env, [RH | RT]) :-
	!,
	rewrite_term(H, CStack, Env, RH),
	rewrite_term(T, CStack, Env, RT).

% operator - flip arguments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(flip, [Term]), CStack, Env, RTerm) :-
	!,
	rewrite_term(Term, CStack, Env, RT),
	flip(RT, RTerm).

rewrite_term_rec(fot(rcp_call, [Term]), _CStack, _Env, rcp_call(Term)) :-
	cg_mode(dynamic),
	!.

rewrite_term_rec(fot(flip, [Term]), CStack, _Env, _RTerm) :-
	!,
	raise_exception(generic_rewriting(unknown, [Term], CStack)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% global constraints rewriting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(geost_constraint, [Objects, Sboxes, Dims]), CStack, Env, RTerm) :-
	!,
	rewrite_list([Objects, Sboxes, Dims], [geost_constraint | CStack], Env,
		     [RObjects, RSboxes, RDims]),
	RTerm =.. [geost_constraint, RObjects, RSboxes, RDims].

% geost with greedy assignment mode options (sp4.0.3-NETWMS version)
rewrite_term_rec(fot(geost_constraint, [Objects, Sboxes, Dims, Options]), CStack, Env, RTerm) :-
	!,
	rewrite_list([Objects, Sboxes, Dims, Options],
		     [geost_constraint | CStack], Env,
		     [RObjects, RSboxes, RDims, ROptions]),
	RTerm =.. [geost_constraint, RObjects, RSboxes, RDims, ROptions].

rewrite_term_rec(fot(fixall, [RFlagVar, RPatterns]), [geost_constraint | _CStack], _Env,
		 fixall(RFlagVar, RPatterns)) :-
	!.

rewrite_term_rec(fot(object, [SidSpec, OriginSpecs]), CStack, Env,
		 object(RSidSpec, ROriginSpecs)) :-
	!,
	rewrite_list([SidSpec, OriginSpecs], CStack, Env, [RSidSpec, ROriginSpecs]).

rewrite_term_rec(fot(FOrder, [Rank]), CStack, Env, RTerm) :-
	memberchk(FOrder, [min, max]),
	!,	
	rewrite_term(Rank, [CStack], Env, RRank),
	RTerm =.. [FOrder, RRank].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generic rewriting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% name(expr, ...,expr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(Functor, Args), CStack, Env, RTerm) :-
	!,
	rewrite_list(Args, [Functor | CStack], Env, RArgs),	
	Term =.. [Functor | RArgs],
	rewrite_term_rec(fot(Term), CStack, Env, RTerm).

rewrite_term_rec(fot(Term), _CStack, _Env, RTerm) :-
	cg_mode(dynamic),
	subterm(^, Term),
	!,
	(	  
	  (
                                %   function(Term, _, _)
                                % ;
	    relation(Term, _, _)
	  ;
	    inlined(Term)
	  )
	->
	  RTerm = Term
	;
	  Term =.. [Functor | Args],
	  RTerm = rcp_att(Functor, Args)
	).

rewrite_term_rec(fot(Term), _CStack, _Env, Term) :-
	subterm(^, Term),
	!.

rewrite_term_rec(fot(Term), _CStack, _Env, Term) :-
	inlined(Term),
	!.

% attribute
%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(Term), CStack, Env, RTerm) :-
	attribute(Term, V),
	!,
	rewrite_term(attribute_term(V), [Term | CStack], Env, RTerm).

rewrite_term_rec(attribute_term(Term), _CStack, _Env, rcp_att(Functor, Args)) :-
	cg_mode(dynamic),
	!,
	Term =.. [Functor | Args].
rewrite_term_rec(attribute_term(Term), CStack, Env, RTerm) :-	
	!,
	rewrite_term(Term, CStack, Env, RTerm).

% % function
% %%%%%%%%%%%%%%%%%%%%%
% rewrite_term_rec(fot(Term), CStack, Env, RTerm) :-
% 	function(Term, Head, Body),
% 	!,
% 	rewrite_term(function_term(Term, Head, Body), CStack, Env, RTerm).
% rewrite_term_rec(function_term(Term, _Head, _Body), _CStack, _Env, Term) :-
% 	cg_mode(dynamic),
% 	!.
% rewrite_term_rec(function_term(Term, Head, Body), CStack, Env, RTerm) :-
% 	!,
% 	Term =.. [Functor | Args],
% 	Head =.. [Functor | FormalArgs],
	
% 	counter_init(local_vars),
% 	env_new(EmptyEnv),
% 	env_add_list(FormalArgs, EmptyEnv, Args, Env1),
% 	substitution(Env1, Body, Term, _, BodySubst),
	
% 	rewrite_term(BodySubst, [Term | CStack], Env, RTerm).

% relation
%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(Term), CStack, Env, RTerm) :-
	relation(Term, Head, Body),
	!,
	rewrite_term(relation_term(Term, Head, Body), CStack, Env, RTerm).
rewrite_term_rec(relation_term(Term, _Head, _Body), _CStack, _Env, Term) :-
	cg_mode(dynamic),
	!.
rewrite_term_rec(relation_term(Term, Head, Body), CStack, Env, RTerm) :-
	!,
	Term =.. [Functor | Args],
	Head =.. [Functor | FormalArgs],
	env_new(EmptyEnv),
	env_add_list(FormalArgs, EmptyEnv, Args, Env1),
        counter_init(local_vars),
	substitution(Env1, Body, Term, _, BodySubst),	    
	rewrite_term(BodySubst, [Term | CStack], Env, RT),
        (
           CStack = [Op | CSubStack]
        ->
           skip_logops(CSubStack, SCStack, SrcTerm),
           insert_label_node(Term, [Op, SrcTerm | SCStack], RT, RTerm)       
        ;  
           RT = RTerm
        ).

% default
%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(Term), _CStack, _Env, rcp_att(Functor, Args)) :-
	cg_mode(dynamic),
	!,
	Term =.. [Functor | Args].

% exception
%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(fot(Term), CStack, _Env, _RTerm) :-
	!,
	raise_exception(generic_rewriting(not_defined, [Term], CStack)).

% fol connectives
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(P and Q, CStack, Env, RTerm) :-
	!,
	rewrite_term(P, [and | CStack], Env, RP),
	rewrite_term(Q, [and | CStack], Env, RQ),
	eval((RP and RQ), RTerm).

rewrite_term_rec(P or Q, CStack, Env, RTerm) :-
	!,
	rewrite_term(P, [or | CStack], Env, RP),
	rewrite_term(Q, [or | CStack], Env, RQ),
	eval(RP or RQ, RTerm).

rewrite_term_rec(not P, CStack, Env, RTerm) :-
	!,
	rewrite_term(P, CStack, Env, RP),
	eval(not RP, RTerm).

rewrite_term_rec(P implies Q, CStack, Env, RTerm) :-
	!,
	rewrite_term(P, CStack, Env, RP),
	rewrite_term(Q, CStack, Env, RQ),
	eval((RP implies RQ), RTerm).

rewrite_term_rec(P equiv Q, CStack, Env, RTerm) :-
	!,
	rewrite_term(P, CStack, Env, RP),
	rewrite_term(Q, CStack, Env, RQ),
	eval((RP equiv RQ), RTerm).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% higher-order functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% let
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(let(X, E, F), CStack, Env, let(AlphaV, V, RAlphaF)) :-
	cg_mode(dynamic),
	!,
	rewrite_term(X, CStack, Env, X0),
	rewrite_term(E, CStack, Env, V),
	alpha_conversion(X0, V, F, AlphaV-AlphaF),
	env_add(AlphaV, Env, AlphaV, NewEnv),
	rewrite_term(AlphaF, [let | CStack], NewEnv, RAlphaF).
rewrite_term_rec(let(V, E, F), CStack, Env, RTerm) :-
	!,
	rewrite_term(V, [let | CStack], Env, Var),
	rewrite_term(E, [let | CStack], Env, Val),
	alpha_conversion(Var, [Val], F, AlphaV-AlphaF),
	env_add(AlphaV, Env, AlphaV, NewEnv),
	subst(AlphaV, Val, AlphaF, InstF),
	rewrite_term(InstF, [let | CStack], NewEnv, RInstF),
	eval(RInstF, RTerm).

% forall
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(forall(V, E, F), CStack, Env, RTerm) :-
	cg_mode(dynamic),
	!,	
	rewrite_term(foldr(V, E, and, 1, F), CStack, Env, RTerm).
rewrite_term_rec(forall(V, E, F), CStack, Env, RTerm) :-
	!,
	rewrite_term(foldr(V, E, and, 1, F), CStack, Env, RTerm).

% exists
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(exists(V, E, F), CStack, Env, RTerm) :-
	cg_mode(dynamic),
	!,
	rewrite_term(foldr(V, E, or, 0, F), CStack, Env, RTerm).
rewrite_term_rec(exists(V, E, F), CStack, Env, RTerm) :-
	!,
	rewrite_term(foldr(V, E, or, 0, F), CStack, Env, RTerm).

% fold right 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rewrite_term_rec(foldr(X, E, O, I, F), CStack, Env,
% 		 foldr(AlphaV, L, O, RI, RF)) :-
% 	icg_mode,
% 	!,
% 	rewrite_term(X, [O | CStack], Env, V),
% 	rewrite_term(E, [O | CStack], Env, L),
% 	rewrite_term(I, [O | CStack], Env, RI),	
% 	alpha_conversion(V, L, F, AlphaV-AlphaF),	
% 	env_add(AlphaV, Env, AlphaV, NewEnv),
% 	rewrite_term(AlphaF, CStack, NewEnv, RF).
rewrite_term_rec(foldr(X, E, O, I, F), CStack, Env, RTerm) :-
	cg_mode(dynamic),
	!,
	rewrite_term(X, CStack, Env, V),
	rewrite_term(E, CStack, Env, L),
	rewrite_term(I, CStack, Env, RI),
	alpha_conversion(V, L, F, AlphaV-AlphaF),
	env_add(AlphaV, Env, AlphaV, NewEnv),
	counter_inc(it_rank, NIt),
	rewrite_term(AlphaF, CStack, NewEnv, RF),	
	first_defunctor(CStack, From),	
	functor(From, FFrom, _),	
	atomize([FFrom, foldr, NIt], '_', HFunctor),
	env_vars_list(Env, EnvVarList),
	Head =.. [HFunctor, cons(AlphaV, var('Tail',0)), list(EnvVarList), O, RI],
	RecCall =.. [HFunctor, var('Tail',0), list(EnvVarList)],
	Body = RF-RecCall,
	RTerm =.. [HFunctor, L, EnvVarList],
	store_statement(Head, Body, foldr).
rewrite_term_rec(foldr(X, E, O, I, F), CStack, Env, RTerm) :-
	!,
	rewrite_term(X, CStack, Env, V),
	rewrite_term(E, CStack, Env, L),
	reduced_to_type_or_excpt(list, E, L, CStack),
	alpha_conversion(V, L, F, AlphaV-AlphaF),
	rewrite_term(foldr_list(AlphaV, L, O, I, AlphaF), [O | CStack], Env, RTerm).

% fold right over lists 
rewrite_term_rec(foldr_list(_, [], _, Identity, _), CStack, Env, RIdentity) :-
	!,
	rewrite_term(Identity, CStack, Env, RIdentity).       
rewrite_term_rec(foldr_list(Var, [Val | List], Operator, Identity, Fol),
		 CStack, Env, RTerm) :-
	!,
	subst(Var, Val, Fol, InstFol),
	rewrite_term(InstFol, CStack, Env, RInstFol),
	eval(RInstFol, ERInstFol),
	rewrite_term(foldr_list(Var, List, Operator, Identity, Fol), CStack, Env, RT),
	composition(Operator, ERInstFol, RT, ERT),
	(
	  is_builtinop(ERT)
	->
	  eval(ERT, RTerm)
	;
	  parse_expr(ERT, PTerm),
	  rewrite_term(PTerm, CStack, Env, RTerm)
	).

% fold left
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rewrite_term_rec(foldl(X, E, O, I, F), CStack, Env,
% 		 foldl(AlphaV, L, O, RI, RF)) :-
% 	icg_mode,
% 	!,
% 	rewrite_term(X, [O | CStack], Env, V),
% 	rewrite_term(E, [O | CStack], Env, L),
% 	rewrite_term(I, [O | CStack], Env, RI),
% 	alpha_conversion(V, L, F, AlphaV-AlphaF),	
% 	env_add(AlphaV, Env, AlphaV, NewEnv),
% 	rewrite_term(AlphaF, CStack, NewEnv, RF).
rewrite_term_rec(foldl(X, E, O, I, F), CStack, Env, RTerm) :-
	cg_mode(dynamic),
	!,
	rewrite_term(X, [O | CStack], Env, V),
	rewrite_term(E, [O | CStack], Env, L),
	rewrite_term(I, [O | CStack], Env, RI),
	alpha_conversion(V, L, F, AlphaV-AlphaF),	
	env_add(AlphaV, Env, AlphaV, NewEnv),
	env_vars_list(Env, EnvVarList),
	counter_inc(it_rank, NIt),
	rewrite_term(AlphaF, CStack, NewEnv, RF),
	first_defunctor(CStack, From),	
	functor(From, FFrom, _),
	atomize([FFrom, foldl, NIt], '_', HFunctor),	
	Head =.. [HFunctor, cons(AlphaV, var('Tail',0)), list(EnvVarList), O],
	RecCall =.. [HFunctor, var('Tail',0), list(EnvVarList)],	
	
	Body = RF-RecCall,
	RTerm =.. [HFunctor, L, RI, EnvVarList],	
	store_statement(Head, Body, foldl).

rewrite_term_rec(foldl(X, E, O, I, F), CStack, Env, RTerm) :-
	!,
	rewrite_term(X, CStack, Env, V),
	rewrite_term(E, CStack, Env, L),
	reduced_to_type_or_excpt(list, E, L, CStack),	    
	alpha_conversion(V, L, F, AlphaV-AlphaF),	
	rewrite_term(foldl_list(AlphaV,L,O,I,AlphaF), CStack, Env, RTerm).

% fold left over lists 
rewrite_term_rec(foldl_list(_, [], _, Identity, _), CStack, Env, RIdentity) :-
	!,
	rewrite_term(Identity, CStack, Env, RIdentity).
rewrite_term_rec(foldl_list(Var, [Val | List], Operator, Identity, Fol),
		 CStack, Env, RTerm) :-
	!,
	subst(Var, Val, Fol, InstFol),
	rewrite_term(InstFol, CStack, Env, RInstFol),
	eval(RInstFol, ERInstFol),
	composition(Operator, Identity, ERInstFol, ERT),
	(
	  is_builtinop(ERT)
	->
	  eval(ERT, RT)
	;
	  parse_expr(ERT, PTerm),
	  rewrite_term(PTerm, CStack, Env, RT)
	),
	rewrite_term(foldl_list(Var, List, Operator, RT, Fol),
		     CStack, Env, RTerm).

% filter 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(filter(P, E), CStack, Env, RTerm) :-
	!,	
	rewrite_term(E, CStack, Env, L),
	reduced_to_type_or_excpt(list, E, L, CStack),
	rewrite_term(filter_list(P, L), CStack, Env, RTerm).

% filter over lists 
rewrite_term_rec(filter_list(_, []), _CStack, _Env, []) :-
	!.       
rewrite_term_rec(filter_list(P, [V | Vs]), CStack, Env, RTerm) :-
	!,
	rewrite_term(filter_list(P, Vs), CStack, Env, RTermSoFar),	
	FilterTerm =.. [P, V],
	rewrite_term(FilterTerm, CStack, Env, FRTerm),
	eval(FRTerm, EvalFRTerm),
	reduced_to_type_or_excpt(boolean, FilterTerm, EvalFRTerm, CStack),
	(
	  EvalFRTerm
	->
	  RTerm = [V | RTermSoFar]
	;
	  RTerm = RTermSoFar
	).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% data structures (record|list|range)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% range 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec([LExpr .. UExpr], CStack, Env, RTerm) :-	
	rewrite_term(LExpr, CStack, Env, L),
	rewrite_term(UExpr, CStack, Env, U),
	!,
	rewrite_term(rcp_range(L, U), CStack, Env, RTerm).
rewrite_term_rec(rcp_range(L, U), _CStack, _Env, [L .. U]) :-
	cg_mode(dynamic),
	!.
rewrite_term_rec(rcp_range(min_integer, max_integer), _CStack, _Env,
		 [min_integer .. max_integer]) :-
	!.
rewrite_term_rec(rcp_range(min_integer, U), _CStack, _Env,
		 [min_integer .. U]) :-
	integer(U),
	!.
rewrite_term_rec(rcp_range(min_integer, U), CStack, _Env, _RTerm) :-	
	!,
	raise_exception(generic_rewriting(not_reduced_to_type(integer),[U],CStack)).
rewrite_term_rec(rcp_range(L, max_integer), _CStack, _Env,
		 [L .. max_integer]) :-	
	integer(L),
	!.
rewrite_term_rec(rcp_range(L, max_integer), CStack, _Env, _RTerm) :-	
	!,
	raise_exception(generic_rewriting(not_reduced_to_type(integer),[L],CStack)).
rewrite_term_rec(rcp_range(L, U), [in | _CStack], _Env, [L .. U]) :-
	integer(L),
	integer(U),
	!.
rewrite_term_rec(rcp_range(L, U), _CStack, _Env, RTerm) :-
	integer(L),
	integer(U),
	!,
	gen_integers(L, U, RTerm).
rewrite_term_rec(rcp_range(L, U), CStack, _Env, _RTerm) :-
	!,
	raise_exception(generic_rewriting(not_reduced_to_type(integer),[[L, U]], CStack)).

% record (association-list)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(rcp_rec(Term), CStack, Env, rcp_rec(RTerm)) :-
	!,
	rewrite_list(Term, CStack, Env, RT),
	(
	  (
	    memberchk(uid = Uid, RT)
	  ;
	    cg_mode(dynamic)
	  )
	->
	  RTerm = RT
	;	  
	  record_uid_table_store(RT, Uid),
	  append(RT, [uid = Uid], RTerm)
	).

% list 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_rec(Term, CStack, Env, RTerm) :-
	is_list(Term),
	rewrite_list(Term, CStack, Env, RTerm).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% list rewriting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_list([], _CStack, _Env, []).
rewrite_list([T | Ts], CStack, Env, [RT | RTs]) :-
	rewrite_term(T, CStack, Env, RT),
	rewrite_list(Ts, CStack, Env, RTs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% statements rewriting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_rewrite_statements :-
	retractall(statement(_, _, _)).

% rewrite constant statement 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_statement(constant(Head, Body)) :-
	!,
	counter_init(local_vars),
                                % write(constant(Head)),nl,
				% counter_init(it_rank),
	env_new(Env),
	rewrite_term(Body, [Head], Env, RBody),
	store_statement(Head, RBody, constant).
                                % 	set_icg_mode,
                                % 	rewrite_term(Body, [Head], Env, RBody1),
                                % 	store_rcp_statement(Head, RBody1, constant),
                                % 	unset_icg_mode.

% rewrite function statement 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rewrite_statement(function(Head, Body)) :-
%    !,
%    counter_init(local_vars),
% 				%counter_init(it_rank),
%    env_new(EmptyEnv),
%    Head =.. [_ | FormalArgs],
%    env_add_list(FormalArgs, EmptyEnv, FormalArgs, Env),
%    rewrite_term(Body, [Head], Env, RBody),
%    store_statement(Head, RBody, function).
%                                 % 	set_icg_mode,
%                                 % 	rewrite_term(Body, [Head], Env, RBody1),
%                                 % 	store_rcp_statement(Head, RBody1, function),
%                                 % 	unset_icg_mode.

% rewrite rule statement 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_statement(rule(Head, Body)) :-
				% counter_init(it_rank),
                                %write(rule(Head)),nl,
   env_new(EmptyEnv),
   Head =.. [_ | FormalArgs],
   env_add_list(FormalArgs, EmptyEnv, FormalArgs, Env),
   rewrite_term(Body, [Head], Env, RBody),
   store_statement(Head, RBody, rule).
                                % 	set_icg_mode,
                                % 	rewrite_term(Body, [Head], Env, RBody1),
                                % 	store_rcp_statement(Head, RBody1, rule),
                                % 	unset_icg_mode.

% rewrite query statement 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rewrite_statement(query(Body)) :-
% 	env_new(EmptyEnv),
% 	set_icg_mode,
% 	rewrite_term(Body, [], EmptyEnv, RBody),
% 	store_rcp_statement(solve_query, RBody, query),
% 	unset_icg_mode.

% rewrite statements
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_statements([]).
rewrite_statements([Def | Defs]) :-
	rewrite_statement(Def),
	rewrite_statements(Defs).

rewrite_statements :-
	findall(constant(H, B), constant(H, B), LConstants),
	findall(rule(H, B), rule(H, B), LRules),
                                %findall(function(H, B), function(H, B), LFunctions),
                                %findall(query(B), query(B), LQuery),
	cg_mode(CGModeBefore),
	set_cg_mode(dynamic),
	retractall(rewrite_term_hash(_, _, _)),        
	rewrite_statements(LConstants),
	rewrite_statements(LRules),
                                %rewrite_statements(LFunctions),
                                %rewrite_statements(LQuery),
	set_cg_mode(CGModeBefore).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% search statements rewriting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_rewrite_statements_search :-
	retractall(statement_search(_, _, _)).

update_costs(CCostList - DCostList, Pred, NewCCostList - NewDCostList) :-
	conjorder_terms(COCrits),
	disjorder_terms(DOCrits),	
	update_costs_rec(CCostList, Pred, COCrits, NewCCostList),
	update_costs_rec(DCostList, Pred, DOCrits, NewDCostList).

update_costs_rec([], _Pred, [], []).
update_costs_rec([Cost | CostTail], Pred, [OCrit | OCrits], [NewCost | NewCostList]) :-
	OCrit =.. [Order, Expr if is(^, Match)],
	functor(Match, Functor, Arity),
	functor(MatchVarArgs, Functor, Arity),	
	(
	  MatchVarArgs = Pred
	->
	  Match =.. [Functor | FormalArgs],
	  Pred =.. [Functor | Args],
	  env_new(EmptyEnv),
	  env_add_list(FormalArgs, EmptyEnv, Args, Env),
	  substitution(Env, Expr, '', _, SubstExpr),
	  apply_order_sign(Order, SubstExpr, NewCost)
	;
	  NewCost = Cost
	),				
	update_costs_rec(CostTail, Pred, OCrits, NewCostList).

apply_order_sign(greatest, SubstExpr, -SubstExpr) :-
	!.
apply_order_sign(least, SubstExpr, SubstExpr).

% rewrite rule statement 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_statement_search(rule(Head, Body)) :-
	!,
	Head =.. [PF | FormalArgs],
	length(FormalArgs, A),
	predicate_costlists(PF/A, CostLists),

	env_new(EmptyEnv),
	env_add_list(FormalArgs, EmptyEnv, FormalArgs, Env),
	env_list(Env, VsPair),
	proj_list_pair_left(VsPair, Vs),
	
	update_costs(CostLists, Head, NewCostLists),
	
	atomize([PF, and], '_', PF_and),
	P_and =.. [PF_and, CostLists | Vs],
	rewrite_term_search(Body, and, NewCostLists, Env, [P_and], RCBody),
	store_statement(P_and, RCBody, rule_search),
	
	atomize([PF, or], '_', PF_or),
	P_or =.. [PF_or, CostLists | Vs],
	rewrite_term_search(Body, or, NewCostLists, Env, [P_or], RDBody),
	store_statement(P_or, RDBody, rule_search).

rewrite_statements_search([]).
rewrite_statements_search([Def | Defs]) :-
	rewrite_statement_search(Def),
	rewrite_statements_search(Defs).

rewrite_statements_search :-
	findall(rule(H, B), rule(H, B), LRules),
	rewrite_statements_search(LRules).

% store statement (to execute)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
store_statement(Head, Body, Type) :-
	statement(Head, Body, Type),
	!.
store_statement(Head, Body, Type) :-
	assertz(statement(Head, Body, Type)).

% store rcp statement (to interpret)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
store_rcp_statement(Head, Body, Type) :-
	rcp_statement(Head, Body, Type),
	!.
store_rcp_statement(Head, Body, Type) :-
	assertz(rcp_statement(Head, Body, Type)).

% store rcp heuristics  (to interpret)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
store_rcp_heuristics(OrderTerms, Type) :-
	rcp_heuristics(Type, OrderTerms),
	!.
store_rcp_heuristics(OrderTerms, Type) :-
	assertz(rcp_heuristics(Type, OrderTerms)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% "type" checking
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% is Term an assoc list ? 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_assoc_list([(_ = _)]).
is_assoc_list([(_ = _) | Assocs]) :-
	is_assoc_list(Assocs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% statements checking and accessing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% propositional term 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% constant_function(Term, Term, Body) :-
% 	!,
% 	constant(Term, Body).

constant_relation(Term, Term, Body) :-
	constant(Term, Body).

% first order term / atom
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function(Term, Head, Body) :- 
% 	functor(Term, Functor, Arity),
% 	functor(Head, Functor, Arity),
% 	function(Head, Body),
% 	!.

relation(Term, Head, Body) :-
   functor(Term, Functor, Arity),
   functor(Head, Functor, Arity),
   rule(Head, Body),
   !.

attribute(Term, Value) :-
	Term =.. [Att, rcp_rec(AssocList)],
	memberchk(Att = Value, AssocList).

inlined(Term) :-
	functor(Term, Functor, Arity),
	lang(inlined_preds, Preds),
	memberchk(Functor/Arity, Preds).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RTerm is of type T or raise exception
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ^
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(_Type, _Term, RTerm, _CStack) :-
	subterm(^, RTerm),
	!.

% boolean
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(boolean, _Term, RTerm, _CStack) :-
	is_bool(RTerm),
	!.

% integer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(integer, _Term, RTerm, _CStack) :-
	integer(RTerm),
	!.

% variable
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(variable, _Term, RTerm, _CStack) :-
	is_var(RTerm),
	!.

% variable or integer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(var_or_int, _Term, RTerm, _CStack) :-
	is_var(RTerm),
	!.
reduced_to_type_or_excpt(var_or_int, _Term, RTerm, _CStack) :-
	integer(RTerm),
	!.

% list
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(list, _Term, RTerm, _CStack) :-
	is_list(RTerm),
	!.


% list of list of variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(list_of_list_of_var, _Term, RTerm, _CStack) :-
	list_of_list_of_var(RTerm),	
	!.

% exception
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
reduced_to_type_or_excpt(Type, Term, _RTerm, CStack) :-
	raise_exception(generic_rewriting(not_reduced_to_type(Type), [Term],
					  CStack)).

list_of_var([]).
list_of_var([H | T]) :-
	is_var(H),
	list_of_var(T).

list_of_list_of_var([]).
list_of_list_of_var([H | T]) :-
	list_of_var(H),
	list_of_list_of_var(T).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% normalization ('not', '1', '0' elimination in fol)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
compop_neg(<, >=).
compop_neg(=<, >).
compop_neg(=, #).
compop_neg(>=, <).
compop_neg(>, =<).
compop_neg(#, =).

normalize_term(Term, Term) :-
	simple(Term),
	!.
normalize_term(node(L, Term), node(L, NTerm)) :-
	!,
	normalize_term(Term, NTerm).
normalize_term(not node(L, Term), node(L, NTerm)) :-
	!,
	normalize_term(not Term, NTerm).
normalize_term(not (P and Q), NTerm) :-
	!,
	normalize_term((not P or not Q), NTerm).
normalize_term(not (P or Q), NTerm) :-
	!,
	normalize_term((not P and not Q), NTerm).
normalize_term(not (not P), NTerm) :-
	!,
	normalize_term(P, NTerm).
normalize_term(not (X in [L..U]),
	       (X in [min_integer .. U1]) or (X in [L1 .. max_integer])) :-
	!,
	eval(L - 1, U1), 
	eval(U + 1, L1).
normalize_term(not P, NTerm) :-
	is_compop(P),
	!,
	P =.. [F | Args],
	compop_neg(F, NF),
	NTerm =.. [NF | Args].
normalize_term(P and 1, NTerm) :-
	!,
	normalize_term(P, NTerm).
normalize_term(1 and P, NTerm) :-
	!,
	normalize_term(P, NTerm).
normalize_term(P or 0, NTerm) :-
	!,
	normalize_term(P, NTerm).
normalize_term(0 or P, NTerm) :-
	!,
	normalize_term(P, NTerm).
normalize_term(Term, NTerm) :-
	Term =.. [Op | Args],
	normalize_term_rec(Args, NArgs),
	NTerm =.. [Op | NArgs].

normalize_term_rec([], []).
normalize_term_rec([A | Args], [NA | NArgs]):-
	normalize_term(A, NA),
	normalize_term_rec(Args, NArgs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% records' unique identifier handling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_record_uid_table :-
	recorded(record_uid_table(Record, Uid),
		 record_uid_table(Record, Uid), Ref),
	!,
	erase(Ref),
	init_record_uid_table.
init_record_uid_table :-
	counter_init(uid).

record_uid_table_store(Record, Uid) :-
	cg_mode(static),
	recorded(record_uid_table(Record, Uid),
		 record_uid_table(Record, Uid), _),
	!.
record_uid_table_store(Record, Uid) :-
	cg_mode(static),
	counter_inc(uid, Uid),
	recordz(record_uid_table(Record, Uid),
		record_uid_table(Record, Uid), _).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% labelled node insertion in and/or trees (reordering purpose)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
insert_label_node(Term, [and | _], RTerm, node(Term, RTerm)) :-
	srch_mode(reordering),
	!.
insert_label_node(Term, [or | _], RTerm, node(Term, RTerm)) :-
	srch_mode(reordering),
	!.
insert_label_node(_Term, _CStack, RTerm, RTerm).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% search term rewriting 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_search(Term, Op, CostLists, Env, Stack, RTerm) :-
	is_conjdisjop(Term),
	!,
	Term =.. [Op0, A, B],
	(
	  Op = Op0
	->
	  RTerm = append(RTermA, RTermB),
	  rewrite_term_search(A, Op, CostLists, Env, Stack, RTermA),
	  rewrite_term_search(B, Op, CostLists, Env, Stack, RTermB)
	;
	  proj_cost_list(Op, CostLists, CostList),
	  env_list(Env, VsPair),
	  proj_list_pair_left(VsPair, Vs),
	  Q =.. [QF | Vs],
	  rewrite_term_search_new_functor(QF),
	  rewrite_term_search(Term, Op0, CostLists, Env, Stack, SubRTerm),
	  store_statement(Q, iter_sort(Op, SubRTerm), search_function),
	  RTerm = CostList - Q
	).
rewrite_term_search(fot(Functor, Args), Op, CostLists, Env, Stack, RTerm) :-
	Term =.. [Functor | Args],
	relation(Term, _, _),
	!,
	rewrite_term_search_list(Args, Op, CostLists, Env, Stack, RArgs),
	rewrite_term_search_predicate(Functor, CostLists, RArgs, Op, Env, RTerm).
rewrite_term_search(foldr(X, E, O, I, F), Op, CostLists, Env, Stack, RTerm) :-
	!,
	rewrite_term_search_new_functor(QF),
	rewrite_term_search_foldr(X, O, I, F, QF, and, Env, Stack),
	rewrite_term_search_foldr(X, O, I, F, QF, or, Env, Stack),
	atomize([QF, Op], '_', QF_Op),
	rewrite_term(E, Stack, Env, E0),
	env_list(Env, VsPair),
	proj_list_pair_left(VsPair, Vs),
	Q =.. [QF_Op, E0, CostLists | Vs],
	RTerm = Q.
rewrite_term_search(ctn(QF, T, Vs), Op, CostLists, _Env, _Stack, RTerm) :-
	!,
	atomize([QF, Op], '_', QF_Op),
	Q =.. [QF_Op, T, CostLists | Vs],
	RTerm = Q.

rewrite_term_search(Expr, Op, CostLists, Env, Stack, RTerm) :-
	rewrite_term(Expr, Stack, Env, RExpr),
	proj_cost_list(Op, CostLists, CostList),
	RTerm = CostList - RExpr.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rewrite_term_search_list([], _Op, _CostLists, _Env, _Stack, []).
rewrite_term_search_list([H | T], Op, CostLists, Env, Stack, [RH | RT]) :-
	rewrite_term_search(H, Op, CostLists, Env, Stack, RH),
	rewrite_term_search_list(T, Op, CostLists, Env, Stack, RT).

rewrite_term_search_predicate(PF, CostLists, RArgs, Op, Env, RTerm) :-
	atomize([PF, Op], '_', PF_Op),
	env_list(Env, VsPair),
	proj_list_pair_left(VsPair, Vs),
	RTerm =.. [PF_Op, CostLists | Vs],
	length(RArgs, A),
	assert(predicate_costlists(PF/A, CostLists)).

rewrite_term_search_new_functor(QF) :-
	counter_inc(search_statement, N),
	atomize([search_subterm, N], '_', QF).

rewrite_term_search_foldr(X, O, I, F, QF, Op, Env, Stack) :-
	atomize([QF, Op], '_', QF_Op),
	init_cost_lists(CostLists),
	env_list(Env, VsPair),
	proj_list_pair_left(VsPair, Vs),
	QNil =.. [QF_Op, [], CostLists | Vs],
	rewrite_term_search(I, Op, CostLists, Env, Stack, RTermQNil),
	store_statement(QNil, RTermQNil, search_function),
	Y = var('Y', 0),
	T = var('T', 0),
	QCons =.. [QF_Op, [Y | T], CostLists | Vs],
	subst(X, Y, F, F0),
	rewrite_term_search(fot(O, [F0, ctn(QF, T, Vs)]), Op, CostLists, Env, Stack, RTermQCons),
	store_statement(QCons, RTermQCons, search_function).


init_cost_list([]).
init_cost_list([undefined | T]) :-
	init_cost_list(T).

init_cost_lists(CCostList - DCostList) :-
	conjorder_terms(COCrits),
	disjorder_terms(DOCrits),
	length(COCrits, NC),
	length(CCostList, NC),
	length(DOCrits, ND),
	length(DCostList, ND),
	init_cost_list(CCostList),
	init_cost_list(DCostList).

proj_cost_list(and, ProjCostList - _, ProjCostList).
proj_cost_list(or, _ - ProjCostList, ProjCostList).	
	

proj_list_pair_left([], []).
proj_list_pair_left([A - _ | T], [A | U]) :-
	proj_list_pair_left(T, U).

proj_list_pair_right([], []).
proj_list_pair_right([_ - B | T], [B | U]) :-
	proj_list_pair_right(T, U).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% includes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- ensure_loaded([order]).
