% Copyright 2008 INRIA, F. Fages & J. Martin
% License GPL
%
% file parser.pl
%

:- use_module(library(codesio)).
:- ensure_loaded([lang]).
:- ensure_loaded([misc]).

:- dynamic code_gen_mode/1,
   constant/2,
   function/2,
   rule/2,
   query/1.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parser 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse(Files) :-
   init_parser,
   parse_files(Files).

init_parser :-
   retractall(rule(_,_)),
   retractall(constant(_,_)),
   retractall(function(_,_)),
   retractall(query(_)),	
   init_rcp_path,	
   init_rcp_dir,
   retractall(conjorder_terms(_)),
   assert(conjorder_terms([])),
   retractall(disjorder_terms(_)),
   assert(disjorder_terms([])).

parse_files([]).
parse_files([File | Files]) :-		
   parse_file(File),
   parse_files(Files).

parse_file(F) :-
   file_open(F, File, Stream),
   format("Reading file \"~w\"... \n", [File]),		
   parse_stream(Stream),
   format("File \"~w\" read.\n", [File]),	
   close(Stream),
   format("File  \"~w\" closed. \n\n", [File]),
   !.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% stream parser (parses head of statements, read parses body)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_stream(Stream) :-
   get_char(Stream, Char),
   (  
      Char == 'end_of_file'
   -> 
      true
   ;  
      Char == '%'
   -> 
      skip_comment(Stream),
      parse_stream(Stream)
   ;  
      Char == '?'
   -> 
      read_term(Stream, Term, [variable_names(VarNames),
                               syntax_errors(error)]),
      name_variables(Term, VarNames, QueryVarNamed), 
      parse_statement(? QueryVarNamed),
      parse_stream(Stream)
   ;  
      Char @>= 'a',
      Char @=< 'z'
   -> 
      parse_stream_functor(Stream, Char, Functor, NextChar),
      (  
         Functor == 'import'
      ->
         (
            NextChar == '('
         ->
            true
         ;
            syntax_error(Stream, NextChar, ' read but import argument excepted')
         ),
         parse_stream_import_arg(Stream, Arg_chars),
         parse_stream_import_dot(Stream),
         atom_chars(Arg, Arg_chars),         
         Term = import(Arg)
      ;
         NextChar == '('
      ->
         parse_stream_args(Stream, Args),
         get_char(Stream, Char0),
         parse_stream_op(Stream, Char0, Op),
         
         read_term(Stream, Body, [variable_names(VarNames),
                                  syntax_errors(error)]), 
         name_variables(Body, VarNames, BodyVarNamed),
         Head =.. [Functor | Args],
         Term =.. [Op, Head, BodyVarNamed]
      ;
         parse_stream_op(Stream, NextChar, Op),
         
         read_term(Stream, Body, [variable_names(VarNames),
                                  syntax_errors(error)]), 
         name_variables(Body, VarNames, BodyVarNamed),
         Head =.. [Functor],
         Term =.. [Op, Head, BodyVarNamed]
      ),
      parse_statement(Term),
      parse_stream(Stream)
   ;  
      blank_char(Char)
   -> 
      parse_stream(Stream)
   ;  
      syntax_error(Stream, Char, ' cannot start a Cream statement')
   ).

parse_stream_import_arg(Stream, Arg_chars) :-
   get_char(Stream, Char),
   (  
      Char = '\''
   -> 
      parse_stream_import_quoted_arg(Stream, Arg_chars)
   ;  
      blank_char(Char)
   -> 
      parse_stream_import_arg(Stream, Arg_chars)
   ;  
      parse_stream_import_arg_in_the_middle(Stream, Char, Arg_chars)
   ).

parse_stream_import_arg_in_the_middle(Stream, Char, Arg_chars) :-
   (  
      Char == 'end_of_file'
   -> 
      syntax_error(Stream, Char, ' was not expected yet, please close your import statement before')
   ;  
      Char == ')'
   -> 
      Arg_chars = []
   ;  
      blank_char(Char)
   -> 
      parse_stream_import_arg_end(Stream)
   ;  
      Arg_chars = [Char | Tail],
      get_char(Stream, Char0),
      parse_stream_import_arg_in_the_middle(Stream, Char0, Tail)
   ).

parse_stream_import_arg_end(Stream) :-
   get_char(Stream, Char),
   (  
      blank_char(Char)
   -> 
      parse_stream_import_arg_end(Stream)
   ;  
      Char == ')'
   -> 
      true
   ;  
      syntax_error(Stream, Char, ' read but ) expected')
   ).

parse_stream_import_quoted_arg(Stream, Arg_chars) :-
   get_char(Stream, Char),
   (  
      Char == 'end_of_file'
   -> 
      syntax_error(Stream, Char, ' was not expected yet, please close your import statement before')
   ;  
      Char == '\''
   -> 
      Arg_chars = [],
      parse_stream_import_arg_end(Stream)
   ;  
      Char == '\\'
   -> 
      get_char(Stream, Char0),
      (  
         Char0 == 'end_of_file'
      -> 
         syntax_error(Stream, Char0, ' was not expected yet')
         ;
         Arg_chars = [Char0 | Tail],
         parse_stream_import_quoted_arg(Stream, Tail)
         )
      ;
      Arg_chars = [Char | Tail],
      parse_stream_import_quoted_arg(Stream, Tail)
      ).

parse_stream_import_dot(Stream) :-
   get_char(Stream, Char),
   (
      Char == '.'
   ->
      true
   ;
      blank_char(Char)
   ->
      parse_stream_import_dot(Stream)
   ;
       syntax_error(Stream, Char, ' read but . was expected')
   ).

parse_stream_functor(Stream, Char, Functor, NextChar) :-
   parse_stream_functor_chars(Stream, Functor_chars, NextChar),
   atom_chars(Functor, [Char | Functor_chars]).

parse_stream_functor_chars(Stream, Functor_chars, NextChar) :-
   get_char(Stream, Char),
   (
      middle_term_char(Char)
   -> 
      Functor_chars = [Char | Tail],
      parse_stream_functor_chars(Stream, Tail, NextChar) 
   ;
      Functor_chars = [],
      NextChar = Char
   ).

parse_stream_args(Stream, Args) :-
   parse_stream_var(Stream, Var_chars, Final),
   atom_chars(Var, Var_chars),
   Args = [var(Var, 0) | Tail],
   (  
      Final == ')'
   -> 
      Tail = []
   ;  
      Final == ','
   -> 
      parse_stream_args(Stream, Tail)
   ).

parse_stream_var(Stream, Var_chars, Final) :-
   get_char(Stream, Char),
   (  
      (  
         Char @>= 'A',
         Char @=< 'Z'
      ;  
         Char == '_'
      )
   -> 
      Var_chars = [Char | Tail],
      parse_stream_var_tail(Stream, Tail, Final)
   ;  
      blank_char(Char)
   -> 
      parse_stream_var(Stream, Var_chars, Final)
   ;  
      syntax_error(Stream, Char, ' cannot appear in an argument name')
   ).

parse_stream_var_tail(Stream, Var_chars, Final) :-
   get_char(Stream, Char),
   (  
      middle_term_char(Char)
   -> 
      Var_chars = [Char | Tail],
      parse_stream_var_tail(Stream, Tail, Final)
   ;  
      Var_chars = [],
      parse_stream_var_end(Stream, Char, Final)
   ).

parse_stream_var_end(Stream, Char, Final) :-
   (  
      blank_char(Char)
   -> 
      get_char(Stream, Char0),
      parse_stream_var_end(Stream, Char0, Final)
   ;  
      (  
         Char == ')'
      ;  
         Char == ','
      )
   -> 
      Final = Char
   ;  
      syntax_error(Stream, Char, ' read but ) or , was expected')
   ).

parse_stream_op(Stream, Char, Op) :-
   (  
      Char = '='
   -> 
      Op = '='
   ;
      Char = '-'
   ->
      get_char(Stream, '-'),
      get_char(Stream, '>'),
      Op = '-->'   
   ;  
      blank_char(Char)
   -> 
      get_char(Stream, Char0),
      parse_stream_op(Stream, Char0, Op)
   ;  
      syntax_error(Stream, Char, ' read but = or --> was expected to introduce a Cream definition')
   ).

blank_char(' ').
blank_char('\t').
blank_char('\n').
blank_char('\r').

skip_comment(Stream) :-
   get_char(Stream, Char),
   (  
      Char == '\n'
   -> 
      true
   ;  
      Char == 'end_of_file'
   -> 
      syntax_error(Stream, Char, ' was not expected')
   ;  
      skip_comment(Stream)
   ).

middle_term_char(Char) :-
   (  
      Char @>= 'A',
      Char @=< 'Z'
   ;  
      Char \= 'end_of_file',
      Char @>= 'a',
      Char @=< 'z'
   ;  
      Char @>= '0',
      Char @=< '9'
   ;  
      Char == '_'
   ).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% statements
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_statement((Head --> Expr)) :-
   !,  
   parse_statement((Head = Expr)).

parse_statement((Head = Expr)) :-
   functor(Head, _, 0),
   !,
   name_anonymous_variables(Expr),
   parse_head(Head, PHead),
   parse_expr(Expr, PExpr),
   variables(PHead, BindedVars),
   name_free_variables(PExpr, BindedVars, PExprOLV),
   assert(constant(PHead, PExprOLV)).

parse_statement((Head = Expr)) :-
   !,
   name_anonymous_variables(Expr),        
   parse_head(Head, PHead),
   parse_expr(Expr, PExpr),
   variables(PHead, BindedVars),
   name_free_variables(PExpr, BindedVars, PExprOLV),
   assert(rule(PHead, PExprOLV)).

parse_statement((? Expr)) :-
   !,
   parse_expr(Expr, PExpr),
   assert(query(PExpr)).

parse_statement(import(FileName)) :-
   atom(FileName),
   atom_concat([FileName, '.rcp'], File),
   !,
   parse_file(File).

parse_statement(Term) :-
   raise_exception(parsing(not_parsed_as(statement), [Term], [])).	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% var 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_var(var(Term, I), var(Term, I)) :-
   !.
parse_var(free_var(Term, A, I), free_var(Term, A, I)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bool
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_bool(Term, Term) :-
   is_bool(Term).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% int
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_int(Term, Term) :-
   integer(Term).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% infinite
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_inf(inf, inf) :-
   !.
parse_inf(sup, sup).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% varint
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_varint(Term, PTerm) :-
   parse_var(Term, PTerm),
   !.
parse_varint(Term, PTerm) :-
   parse_int(Term, PTerm). 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% varbool
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_varbool(Term, PTerm) :-
   parse_var(Term, PTerm),
   !.
parse_varbool(Term, PTerm) :-
   parse_bool(Term, PTerm).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ident
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_ident(Term, Term) :-
   is_ident(Term).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_name(Term, PTerm) :-
   parse_ident(Term, PTerm),
   !.
parse_name(Name:Ident, PName:PIdent) :-
   parse_name(Name, PName),
   parse_ident(Ident, PIdent).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% criterion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_fcrit(C, PC) :-
   parse_cond(C, PC).

                                % C =.. [Functor | Args],
                                % length(Args, Arity),
                                % lang(folordercrit_preds, Preds),
                                % memberchk(Functor/Arity, Preds),
                                % !,
                                % parse_expr(Args, PArgs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% head
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ident[(var, ..., var)]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_head(Term, PTerm) :-
   parse_ident(Term, PTerm),
   !.
parse_head(Term, PTerm) :-
   compound(Term),
   \+ is_list(Term),
   Term =.. [F | Args],	
   parse_ident(F, PF),
   parse_list_var(Args, PArgs),
   PTerm =.. [PF | PArgs].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% expr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% varint 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, PTerm) :-
   parse_varint(Term, PTerm),
   !.

% special atoms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(^, ^) :-
   !.

% infinite
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, PTerm) :-
   parse_inf(Term, PTerm),
   !.

% range
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr([L..U], [PL..PU]) :-
   !,
   parse_expr(L, PL),
   parse_expr(U, PU).

% list 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, PTerm) :- 
   is_list(Term),
   !,
   parse_list(Term, PTerm).

% record 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, rcp_rec(PTerm)) :- 
   parse_record(Term, PTerm),
   !.

parse_expr(rcp_rec(Term), rcp_rec(PTerm)) :-
   !,
   parse_record(Term, PTerm).

% let
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(let(X, E, F), let(V, L, Expr)) :-
   !,
   parse_or_excpt(var, X, V),
   parse_or_excpt(expr, E, L),
   parse_or_excpt(expr, F, Expr).

% map 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(map(X, E, F), PTerm) :-
   !,
   parse_expr(foldr(X, E, cons, [], F), PTerm).

% fold right (expr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(foldr(X, E, O, N, F), foldr(V, L, O, Id, Expr)) :-
   !,
   parse_or_excpt(var, X, V),
   parse_or_excpt(expr, E, L),
   parse_or_excpt(expr, F, Expr),
   parse_or_excpt(expr, N, Id).

% fold left (expr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(foldl(X, E, O, N, F), foldl(V, L, O, Id, Expr)) :-
   !,
   parse_or_excpt(var, X, V),
   parse_or_excpt(expr, E, L),
   parse_or_excpt(expr, F, Expr),
   parse_or_excpt(expr, N, Id).

% filter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(filter(P,E), filter(PP, PE)) :-
   !,
   parse_or_excpt(name, P, PP),
   parse_or_excpt(expr, E, PE).

% expr arithop expr 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, PTerm) :-
   is_arithop(Term),
   !,
   Term =.. [Op, ExprL, ExprR],
   parse_expr(ExprL, PExprL),
   parse_expr(ExprR, PExprR),
   PTerm =.. [Op, PExprL, PExprR].

% arithop(expr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, PTerm) :-
   is_arithop(Term),
   Term =.. [Op, Expr],
   parse_expr(Expr, PExpr),
   PTerm =.. [Op, PExpr].

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fol
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% varbool
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, PTerm) :- 
   parse_varbool(Term, PTerm),
   !.

% in 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr((V in E), (PV in PE)) :-
   !,
   parse_expr(V, PV),
   parse_expr(E, PE).

% let
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(let(V, E, F), PTerm) :-
   !,
   parse_expr(quant(let, [V, E, F]), PTerm).

% forall
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(forall(V, E, F), PTerm) :-
   !,
   parse_expr(quant(forall, [V, E, F]), PTerm).

% exists
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(exists(V, E, F), PTerm) :-
   !,
   parse_expr(quant(exists, [V, E, F]), PTerm).

% fol quantifiers (let | exists | forall)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(quant(Q, [X, E, F]), PTerm) :-
   !,
   parse_or_excpt(var, X, V),
   parse_or_excpt(expr, E, L),
   parse_or_excpt(expr, F, Expr),
   PTerm =.. [Q, V, L, Expr].

% fol connectives 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr((not forall(X, E, F)), PTerm) :-
   !,
   parse_expr(exists(X, E, (not F)), PTerm).

parse_expr((not exists(X, E, F)), PTerm) :-
   !,
   parse_expr(forall(X, E, (not F)), PTerm).

parse_expr((not P), (not PP)) :-
   !,
   parse_expr(P, PP).

parse_expr((P and Q), (PP and PQ)) :-
   !,
   parse_expr(P, PP),
   parse_expr(Q, PQ).

parse_expr((P or Q), (PP or PQ)) :-
   !,
   parse_expr(P, PP),
   parse_expr(Q, PQ).

parse_expr((P implies Q), (PP implies PQ)) :-
   !,
   parse_expr(P, PP),
   parse_expr(Q, PQ).

parse_expr((P equiv Q), (PP equiv PQ)) :-
   !,
   parse_expr(P, PP),
   parse_expr(Q, PQ).

% name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, zot(PTerm)) :-
   parse_name(Term, PTerm),
   !.

% (expr compop expr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, PTerm) :-
   compound(Term),
   is_compop(Term),
   !,
   Term =.. [Op, ExprL, ExprR],
   parse_expr(ExprL, PExprL),
   parse_expr(ExprR, PExprR),
   PTerm =.. [Op, PExprL, PExprR].

% fol ordering (conjunct | disjunct)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, fot(Functor, [PCritList])) :-
   Term =.. [Functor, CritList],	
   lang(folorder_preds, Preds),
   memberchk(Functor/1, Preds),
   is_list(CritList),
   !,   
   parse_list_fcrit(CritList, PCritList).

% {minimize, maximize}(Goal, Cost)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(minimize(search(Term), Cost), fot(minimize, PArgs)) :-
   !,	
   parse_list([search(Term), Cost], PArgs).
parse_expr(minimize(Term, Cost), fot(minimize, PArgs)) :-
   !,	
   parse_list([search(Term), Cost], PArgs).

parse_expr(maximize(search(Term), Cost), fot(maximize, PArgs)) :-
   !,	
   parse_list([search(Term), Cost], PArgs).
parse_expr(maximize(Term, Cost), fot(maximize, PArgs)) :-
   !,	
   parse_list([search(Term), Cost], PArgs).

% name(expr, ..., expr)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_expr(Term, fot(PFunctor,PArgs)) :-
   compound(Term),
   !,
   Term =.. [Functor | Args],
   parse_name(Functor, PFunctor),
   parse_list(Args, PArgs).

% conditional expression
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_cond((least(Expr) if TExpr), (least(PExpr) if PTExpr)) :-
   !,
   parse_expr(Expr, PExpr),
   parse_test(TExpr, PTExpr).
parse_cond((greatest(Expr) if TExpr), (greatest(PExpr) if PTExpr)) :-
   !,
   parse_expr(Expr, PExpr),
   parse_test(TExpr, PTExpr).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% list of ordered pairs 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_list([], []).
parse_list([(K = V) | Ts], [(PK = PV) | PTs]) :-
   parse_name(K, PK),
   parse_expr(V, PV),	
   parse_list(Ts, PTs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% list 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_list([], []).
parse_list([T | Ts], [PT | PTs]) :-
   parse_expr(T, PT),
   parse_list(Ts, PTs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% list of variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_list_var([], []).
parse_list_var([T | Ts], [PT | PTs]) :-
   parse_var(T, PT),
   parse_list_var(Ts, PTs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% list of criteria (on conjuncts or disjuncts)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_list_fcrit([], []).
parse_list_fcrit([C | Cs], [PC | PCs]) :-
   parse_fcrit(C, PC),
   parse_list_fcrit(Cs, PCs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% record 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_record(Term, PTerm) :-
   Term =.. [{}, (OrderedPairSeq)],
   parse_orderedpair_seq((OrderedPairSeq), PTerm).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sequence of ordered pairs 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_orderedpair_seq((K = V), [(PK = PV)]) :-
   !,
   parse_name(K, PK),
   parse_expr(V, PV).
parse_orderedpair_seq(Term, [(PK = PV) | PTs]) :-
   Term =.. [',', (K = V), Comps],	
   parse_name(K, PK),
   parse_expr(V, PV),	
   parse_orderedpair_seq(Comps, PTs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_test((LExpr is RExpr), is(LExpr, RExpr)) :-
   parse_expr(LExpr, _PLExpr),
   parse_expr(RExpr, _PRExpr).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parse or raise exception
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% int
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_or_excpt(int, Term, PTerm) :-
   !,
   parse_int(Term, PTerm).

% var
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_or_excpt(var, Term, PTerm) :-
   !,
   parse_var(Term, PTerm).

% name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_or_excpt(name, Term, PTerm) :-
   !,
   parse_name(Term, PTerm).

% expr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parse_or_excpt(expr, Term, PTerm) :-
   !,
   parse_expr(Term, PTerm).

parse_or_excpt(Type, Term, _PTerm) :-
   raise_exception(parsing(not_parsed_as(Type), [Term], [])).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % "type" checking 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% is Term a boolean ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_bool(Term) :-
   is_true(Term).
is_bool(Term) :-
   is_false(Term).

is_true(true).
is_true(1).

is_false(false).
is_false(0).

% is Term a variable ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_free_var(free_var(_, _, _)).

is_var(var(_, _)).
is_var(free_var(_, _, _)).

% is Term an anonymous variable ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_anonym_var(Term) :-
   is_free_var(Term),
   arg(1, Term, Name),	
   is_underscore(Name).

% is Term an identifier ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_ident(Term) :-
   atom(Term),
   is_lower_case(Term).

% is Term a range ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_range([_ .. _]).

% is Term an arithmetic operator ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_arithop(Term) :-
   functor(Term, Functor, Arity),
   lang(arith_ops, Ops),
   memberchk(Functor/Arity, Ops).

% is Term a comparison operator ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_compop(Term) :-
   functor(Term, Functor, Arity),
   lang(comp_ops, CompOps),
   memberchk(Functor/Arity, CompOps).

% is Term a logic operator ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_logop(Term) :-
   functor(Term, Functor, Arity),
   lang(logic_ops, LogOps),
   memberchk(Functor/Arity, LogOps).

% is Term a logic conjunction or disjunction ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_conjdisjop(Term) :-
   functor(Term, Functor, Arity),
   lang(conjdisj_ops, CDOps),
   memberchk(Functor/Arity, CDOps).

% is Term a list operator ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_listop(Term) :-
   functor(Term, Functor, Arity),
   lang(list_ops, Ops),
   memberchk(Functor/Arity, Ops).

% is Term an arithmetic operator ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_builtinop(Term) :-
   functor(Term, Functor, Arity),		
   lang(builtins, Ops),
   memberchk(Functor/Arity, Ops).

% is Term an optim. pred ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_optim(Term) :-
   functor(Term, Functor, Arity),
   lang(optim_preds, Preds),
   memberchk(Functor/Arity, Preds).

% is Term a variable binder ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_binder(Term) :-
   lang(bind_preds, BindPreds),
   memberchk(Term, BindPreds).

% is Term inlined ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
is_inlined(Term) :-
   functor(Term, Functor, Arity),	
   lang(inlined_preds, Preds),
   memberchk(Functor/Arity, Preds).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% utilities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% r2cp sources path initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_rcp_path :-
   rcp_path(_),
   !.
init_rcp_path :-
   environ('RCPPATH', RCPPATH),
   set_rcp_path(RCPPATH).

% r2cp installation path initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_rcp_dir :-
   environ('RCPDIR', RCPDIR),
   !,
   set_rcp_dir(RCPDIR).
init_rcp_dir :-
   rcp_dir(_).

% open file from rcp_path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
file_open(F, File, Stream) :-
   rcp_path(RcpPath),
   split_string(RcpPath, ':', PathSplitted),
   file_open(F, PathSplitted, File, Stream).

file_open(F, [], _, _) :-
   open(F, read, _),
   !.
file_open(F, [Path | TailPath], File, Stream) :-
   on_exception(_Error,
                (  
                   atom_concat([Path, '/', F], F0),
                   format("Trying to open file \"~w\"... \n", [F0]),
                   open(F0, read, Stream),
                   format("Opening file \"~w\"... \n", [F0]),
                   File = F0
                ),
                (  
                   file_open(F, TailPath, File, Stream)
                )
               ).

