:- use_module(library(between)).
:- use_module(library(lists)).
:- use_module(library(clpfd)).
:- ensure_loaded(geosr).

% Each box is modelled as obj(L, W, H)
% Boxes are placed in containers of the same sizes
% The objective is to minimize the number of containers
% Data are checked in such away they fit in the containers
% Example : if the length of the item is greater than the container length
%           then take the container length as the maximum size
%           Same control for the width and the height

% Total volume = 7399713140!!

place_n(N) :-
	Gabarit = [1200,800,1500],
	GabaritN = [1200,800,1500,6],
	findall(SortKey-obj(L,W,H), clipped(L,W,H,SortKey,Gabarit,1), Objs1),
	keysort(Objs1, Objs2),
	prefix_length(Objs2, Objs2b, N),
	statistics(runtime,_),
	keys_and_values(Objs2b, _, Objs3),
	keys_and_values(Pairs1, Objs3, _), !,
	keysort(Pairs1, Pairs2),
	keyclumped(Pairs2, Groups),
	fix_shapes(Groups, 0),
	diffs_objects(Pairs1, Objects, GabaritN, 0),
	diffs_shapes(Groups, Shapes),
	numlist(1, N, OidList),
	geosr(Objects,
	      Shapes,
	      [(origin(O1,S1,D) ---> O1^x(D)+S1^t(D)),
	       (end(O1,S1,D) ---> O1^x(D)+S1^t(D)+S1^l(D)),
	       (overlap_sboxes(Dims, O1, S1, O2, S2) --->
		   forall(D, Dims,
			  end(O1,S1,D) #> origin(O2,S2,D) #/\
			  end(O2,S2,D) #> origin(O1,S1,D))),
	       (oversize(O1,O2,S1,S2,D) ---> % amount by which two sboxes overlap in dim D
		   max(max(origin(O1,S1,D),origin(O2,S2,D)) - min(origin(O1,S1,D),origin(O2,S2,D)),
		       max(end(O1,S1,D),   end(O2,S2,D)) -    min(end(O1,S1,D),   end(O2,S2,D)))),
	       (gravity(O1,Os) ---> % O1 is either on the floor on sitting on some other box
		   forall(S1,sboxes([O1^sid]),
			  (origin(O1,S1,4) #= 0 #\/
			   exists(O2,Os,
				  O1^oid#\=O2^oid #/\
				  exists(S2,sboxes([O2^sid]),
					 overlap_sboxes([1,2,3],O1,S1,O2,S2) #/\
				         origin(O1,S1,4) #= end(O2,S2,4)))))),
	       forall(O1,objects(OidList),
		      forall(S1,sboxes([O1^sid]),
			     forall(O2,objects(OidList),
				    O1^oid #< O2^oid #=>
				    forall(S2,sboxes([O2^sid]),
					   overlap_sboxes([1,2,3],O1,S1,O2,S2) #=> 
					   #\ overlap_sboxes([4],O1,S1,O2,S2) #/\
					   oversize(O1,O2,S1,S2,2) #=< 10 #/\
					   oversize(O1,O2,S1,S2,3) #=< 10))))]),
	used_memory(Mem1),
	object_vars(Objects, Vars, []),
	labeling([], Vars),
	used_memory(Mem3),
	Mem2 is Mem3-Mem1,
	statistics(runtime,[_,Time]),
	format('N=~d, static memory=~d bytes, dynamic memory=~d bytes, time=~d msec.\n', [N,Mem1,Mem2,Time]), !.
	% report(Pairs1, Objects).

used_memory(Mem) :-
	garbage_collect,
	statistics(global_stack, [G,_]),
	statistics(local_stack, [L,_]),
	statistics(trail, [T,_]),
	statistics(choice, [C,_]),
	Mem is G+L+T+C.

object_vars([]) --> [].
object_vars([object(_,_,[A,B,C,D])|L]) --> [A,B,C,D],
	object_vars(L).


fixall(NbUsed, Scale) :-
	% Model of containers
	% Objets are packed in containers
	% Each object is modelled with 4 decision variables {T, Y, X, Z}
	% T is the container and it ranges from 1 to the number of items
	% MaxLength = 1200,
	% MaxWidth = 800,
	% MaxHeight = 1500,
	Gabarit = [1200,800,1500],
	scale_gabarit(Gabarit, ScaledGabarit, Scale),
	findall(SortKey-obj(L,W,H), clipped(L,W,H,SortKey,ScaledGabarit,Scale), Objs1),
	keysort(Objs1, Objs2),
	keys_and_values(Objs2, _, Objs3),
	keys_and_values(Pairs1, Objs3, _),
	keysort(Pairs1, Pairs2),
	keyclumped(Pairs2, Groups),
	fix_shapes(Groups, 0),
	NbCont in 0..50,
	indomain(NbCont),
	fixall(Pairs1, Groups, Objects, ScaledGabarit, NbCont, NbUsed),
	report(Pairs1, Objects),
	true.

fixall(Pairs, Groups, Objects, Gabarit, NbCont, NbUsed) :-
	N0 is NbCont-1,
	append(Gabarit, [N0], GabaritN),
	diffs_objects(Pairs, Objects, GabaritN, 0),
	diffs_shapes(Groups, Shapes),
	% object_dvars(Objects, Ts, 1),
	% graph_colors(Ts, -1),
	geost(Objects, Shapes, [cumulative(true),longest_hole(true),parconflict(true),visavis(true),fixall(1,[object(_,min(1),[min(2),min(3),min(4),min(5)])])]),
	object_containers(Objects, Containers),
	max_member(Max, Containers),
	NbUsed is Max+1.

object_containers([], []).
object_containers([object(_,_,[X|_])|Os], [X|Xs]) :-
	object_containers(Os, Xs).
	

top(Pairs, Groups, Objects, Gabarit, N) :-
	fd_max(N, Nmax),
	N0 is Nmax-1,
	append(Gabarit, [N0], GabaritN),
	diffs_objects(Pairs, Objects, GabaritN, 0),
	diffs_shapes(Groups, Shapes),
	% object_dvars(Objects, Ts, 1),
	% graph_colors(Ts, -1),
	geost(Objects, Shapes, [fixall(1)]),
	search(Objects, 0, N).

search([], N, N) :- !.
search(Objs0, N0, N) :-
	place(Objs0, N0, Objs),
	N1 is N0+1,
	search(Objs, N1, N).

place([], _, []).
place([Obj|Objs0], T, Objs) :-
	Obj = object(_,_,[T,W,L,H]),
	fd_min(W, W),
	fd_min(L, L),
	fd_min(H, H), !,
	place(Objs0, T, Objs).
place([X|Objs0], T, [X|Objs]) :-
	place(Objs0, T, Objs).
	

graph_colors([], _).
graph_colors([X|Xs], Max0) :-
	X #=< Max0+1,
	Max #= max(Max0,X),
	graph_colors(Xs, Max).

diffs_objects([], [], _, _).
diffs_objects([obj(L,W,H)-SID|Pairs], [object(OID,SID,[T,Y,X,Z])|Objects], Gabarit, OID0) :-
	Gabarit = [Length,Width,Height,Cont],
	OID is OID0+1,
	Xmax is Length-L,
	Ymax is Width-W,
	Zmax is Height-H,
	Tmax is Cont-1,
	X in 0..Xmax,
	Y in 0..Ymax,
	Z in 0..Zmax,
	T in 0..Tmax,
	diffs_objects(Pairs, Objects, Gabarit, OID).

diffs_shapes([], []).
diffs_shapes([obj(L,W,H)-[SID|_]|Groups], [sbox(SID,[0,0,0,0],[1,W,L,H])|Shapes]) :-
	diffs_shapes(Groups, Shapes).

object_dvars([], [], [], [], _).
object_dvars([Obj-_|Pairs], [object(_,_,Origin)|Objects], [X|Xs], [L|Ls], D) :-
	arg(D, Obj, L),
	nth1(D, Origin, X),
	object_dvars(Pairs, Objects, Xs, Ls, D).
	
object_dvars([], [], _).
object_dvars([object(_,_,Origin)|Objects], [X|Xs], D) :-
	nth1(D, Origin, X),
	object_dvars(Objects, Xs, D).

fix_shapes([], _).
fix_shapes([_-Values|Groups], S0) :-
	S is S0+1,
	domain(Values, S, S),
	fix_shapes(Groups, S).

ensure_cps([], _, _).
ensure_cps([X|Xs], [N|Ns], W) :-
	ensure_cp(X, N, W),
	ensure_cps(Xs, Ns, W).

ensure_cp(X, Len, W) :-
	fd_min(X, Min),
	fd_max(X, Max),
	Mid is (Min+Max)>>1,
	(   Max-Min < Len*W -> true
	;   X in Min..Mid,
	    ensure_cp(X, Len, W)
	;   Mid1 is Mid+1,
	    X in Mid1..Max,
	    ensure_cp(X, Len, W)
	).

	
scale_gabarit([L1,W1,H1], [L2,W2,H2], S) :-
	L2 is L1//S,
	W2 is W1//S,
	H2 is H1//S.

clipped(L1, W1, H1, key(NVol,NW,NL,NH), [Lm,Wm,Hm], Scale) :-
	obj(L, W, H),
	L1 is min((L-1)//Scale+1,Lm),
	W1 is min((W-1)//Scale+1,Wm),
	H1 is min((H-1)//Scale+1,Hm),
	NW is -W1,
	NL is -L1,
	NH is -H1,
	NVol is NW*NL*NH.

report([], []).
report([Obj-_|Pairs], [object(_,_,[T,Y,X,Z])|Objects]) :-
	format('~q placed at [T,X,Y,Z]=~q\n', [Obj,[T,X,Y,Z]]),
	report(Pairs, Objects).



:- dynamic obj/3.
obj(300,155,130).
obj(300,155,130).
obj(300,155,130).
obj(300,155,130).
obj(275,190,100).
obj(275,190,100).
obj(275,190,100).
obj(210,150,155).
obj(210,150,155).
obj(245,110,110).
obj(270,145,110).
obj(270,145,110).
obj(270,145,110).
obj(377,260,157).
obj(380,260,165).
obj(720,397,50).
obj(275,260,150).
obj(275,260,150).
obj(275,260,150).
obj(275,260,150).
obj(275,260,150).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(860,45,50).
obj(583,547,50).
obj(670,170,165).
obj(300,170,100).
obj(300,170,100).
obj(300,170,100).
obj(300,170,100).
obj(300,170,100).
obj(300,170,100).
obj(300,170,100).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(580,340,340).
obj(377,260,157).
obj(377,260,157).
obj(370,85,60).
obj(370,85,60).
obj(370,85,60).
obj(370,85,60).
obj(370,85,60).
obj(370,85,60).
obj(380,260,165).
obj(380,260,165).
obj(583,547,50).
obj(583,547,50).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(580,340,340).
obj(580,340,340).
obj(580,340,340).
obj(580,340,340).
obj(580,340,340).
obj(580,340,340).
obj(580,340,340).
obj(580,340,340).
obj(377,260,157).
obj(377,260,157).
obj(520,595,170).
obj(520,170,200).
obj(520,170,200).
obj(520,170,200).
obj(520,170,200).
obj(520,170,200).
obj(920,200,200).
obj(920,200,200).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(580,340,340).
obj(580,340,340).
obj(580,340,340).
obj(377,260,157).
obj(520,595,170).
obj(520,595,170).
obj(380,260,165).
obj(720,1091,50).
obj(520,490,157).
obj(520,490,157).
obj(520,490,157).
obj(583,547,50).
obj(583,547,50).
obj(583,547,50).
obj(583,547,50).
obj(583,547,50).
obj(583,547,50).
obj(583,547,50).
obj(670,170,165).
obj(520,170,200).
obj(910,200,200).
obj(910,200,200).
obj(920,200,200).
obj(920,200,200).
obj(440,350,265).
obj(440,350,265).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(670,170,165).
obj(910,200,200).
obj(910,200,200).
obj(910,200,200).
obj(910,200,200).
obj(910,200,200).
obj(910,200,200).
obj(910,200,200).
obj(910,200,200).
obj(650,200,200).
obj(650,200,200).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(440,350,265).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1500,150,150).
obj(1505,155,50).
obj(1505,155,50).
obj(1505,155,50).
obj(1505,155,50).
obj(1505,155,50).
obj(1505,155,50).
obj(1505,155,50).
obj(1505,155,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(1000,200,200).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(500,425,155).
obj(720,20,20).
obj(720,20,20).
obj(1005,120,80).
obj(1005,155,45).
obj(1005,155,45).
obj(1005,155,45).
obj(1005,155,45).
obj(1005,155,45).
obj(1005,155,45).
obj(610,140,140).
obj(1200,90,90).
obj(90,85,50).
obj(90,85,50).
obj(90,85,50).
obj(90,85,50).
obj(90,85,50).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(490,100,100).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,25).
obj(2000,150,25).
obj(2000,150,25).
obj(2000,150,25).
obj(250,250,100).
obj(250,250,100).
obj(1500,150,25).
obj(1500,150,25).
obj(245,115,110).
obj(245,115,110).
obj(245,115,110).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(2000,150,50).
obj(610,140,140).
obj(610,140,140).
obj(480,420,125).
obj(670,395,130).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).
obj(3000,40,40).

end_of_file.

/*
| ?- statistics(runtime,_), top(N,1), statistics(runtime,[T1,T2]).
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,0,0]
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,0,340]
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,0,680]
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,0,1020]
obj(580,340,340) placed at [T,X,Y,Z]=[0,580,0,0]
obj(580,340,340) placed at [T,X,Y,Z]=[0,580,0,340]
obj(580,340,340) placed at [T,X,Y,Z]=[0,580,0,680]
obj(580,340,340) placed at [T,X,Y,Z]=[0,580,0,1020]
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,340,0]
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,340,340]
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,340,680]
obj(580,340,340) placed at [T,X,Y,Z]=[0,0,340,1020]
obj(520,595,170) placed at [T,X,Y,Z]=[1,0,0,0]
obj(520,595,170) placed at [T,X,Y,Z]=[1,0,0,170]
obj(520,595,170) placed at [T,X,Y,Z]=[1,0,0,340]
obj(440,350,265) placed at [T,X,Y,Z]=[0,580,340,0]
obj(440,350,265) placed at [T,X,Y,Z]=[0,580,340,265]
obj(440,350,265) placed at [T,X,Y,Z]=[0,580,340,530]
obj(440,350,265) placed at [T,X,Y,Z]=[0,580,340,795]
obj(440,350,265) placed at [T,X,Y,Z]=[0,580,340,1060]
obj(440,350,265) placed at [T,X,Y,Z]=[1,0,0,510]
obj(440,350,265) placed at [T,X,Y,Z]=[1,0,0,775]
obj(440,350,265) placed at [T,X,Y,Z]=[1,0,0,1040]
obj(440,350,265) placed at [T,X,Y,Z]=[1,440,0,510]
obj(440,350,265) placed at [T,X,Y,Z]=[1,440,0,775]
obj(440,350,265) placed at [T,X,Y,Z]=[1,440,0,1040]
obj(440,350,265) placed at [T,X,Y,Z]=[1,520,0,0]
obj(440,350,265) placed at [T,X,Y,Z]=[1,0,350,510]
obj(440,350,265) placed at [T,X,Y,Z]=[1,0,350,775]
obj(440,350,265) placed at [T,X,Y,Z]=[1,0,350,1040]
obj(440,350,265) placed at [T,X,Y,Z]=[1,440,350,510]
obj(440,350,265) placed at [T,X,Y,Z]=[1,440,350,775]
obj(440,350,265) placed at [T,X,Y,Z]=[1,440,350,1040]
obj(440,350,265) placed at [T,X,Y,Z]=[1,520,350,0]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,0,0]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,0,265]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,0,530]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,0,795]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,0,1060]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,0,0]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,0,265]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,0,530]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,0,795]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,0,1060]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,350,0]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,350,265]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,350,530]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,350,795]
obj(440,350,265) placed at [T,X,Y,Z]=[2,0,350,1060]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,350,0]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,350,265]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,350,530]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,350,795]
obj(440,350,265) placed at [T,X,Y,Z]=[2,440,350,1060]
obj(440,350,265) placed at [T,X,Y,Z]=[3,0,0,0]
obj(520,490,157) placed at [T,X,Y,Z]=[1,0,0,1305]
obj(520,490,157) placed at [T,X,Y,Z]=[1,520,0,265]
obj(520,490,157) placed at [T,X,Y,Z]=[1,520,0,1305]
obj(1000,200,200) placed at [T,X,Y,Z]=[1,0,595,265]
obj(920,200,200) placed at [T,X,Y,Z]=[3,0,0,265]
obj(920,200,200) placed at [T,X,Y,Z]=[3,0,0,465]
obj(920,200,200) placed at [T,X,Y,Z]=[3,0,0,665]
obj(920,200,200) placed at [T,X,Y,Z]=[3,0,0,865]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,0,1065]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,0,1265]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,200,265]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,200,465]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,200,665]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,200,865]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,200,1065]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,200,1265]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,350,0]
obj(910,200,200) placed at [T,X,Y,Z]=[3,0,400,200]
obj(670,395,130) placed at [T,X,Y,Z]=[0,0,0,1360]
obj(500,425,155) placed at [T,X,Y,Z]=[0,670,340,1325]
obj(500,425,155) placed at [T,X,Y,Z]=[2,0,0,1325]
obj(500,425,155) placed at [T,X,Y,Z]=[2,500,0,1325]
obj(500,425,155) placed at [T,X,Y,Z]=[4,0,0,0]
obj(500,425,155) placed at [T,X,Y,Z]=[4,0,0,155]
obj(500,425,155) placed at [T,X,Y,Z]=[4,0,0,310]
obj(500,425,155) placed at [T,X,Y,Z]=[4,0,0,465]
obj(500,425,155) placed at [T,X,Y,Z]=[4,0,0,620]
obj(500,425,155) placed at [T,X,Y,Z]=[4,0,0,775]
obj(500,425,155) placed at [T,X,Y,Z]=[4,0,0,930]
obj(720,800,50) placed at [T,X,Y,Z]=[4,0,0,1085]
obj(1200,150,150) placed at [T,X,Y,Z]=[1,0,490,1305]
obj(1200,150,150) placed at [T,X,Y,Z]=[1,0,640,1305]
obj(1200,150,150) placed at [T,X,Y,Z]=[2,0,425,1325]
obj(1200,150,150) placed at [T,X,Y,Z]=[2,0,575,1325]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,400,400]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,400,550]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,400,700]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,400,850]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,400,1000]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,400,1150]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,400,1300]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,0]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,400]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,550]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,700]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,850]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,1000]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,1150]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,550,1300]
obj(1200,150,150) placed at [T,X,Y,Z]=[3,0,600,150]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,0,1135]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,0,1285]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,150,1135]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,150,1285]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,300,1135]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,300,1285]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,425,0]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,425,150]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,425,300]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,425,450]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,425,600]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,425,750]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,425,900]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,450,1135]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,450,1285]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,575,0]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,575,150]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,575,300]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,575,450]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,575,600]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,575,750]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,575,900]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,600,1135]
obj(1200,150,150) placed at [T,X,Y,Z]=[4,0,600,1285]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,0]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,150]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,300]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,450]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,600]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,750]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,900]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,1050]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,1200]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,0,1350]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,0]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,150]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,300]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,450]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,600]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,750]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,900]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,1050]
obj(1200,150,150) placed at [T,X,Y,Z]=[5,0,150,1200]
obj(650,200,200) placed at [T,X,Y,Z]=[3,440,0,0]
obj(650,200,200) placed at [T,X,Y,Z]=[4,500,0,0]
obj(480,420,125) placed at [T,X,Y,Z]=[4,500,0,200]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,0,325]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,0,490]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,0,655]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,0,820]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,170,325]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,170,490]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,170,655]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,170,820]
obj(670,170,165) placed at [T,X,Y,Z]=[4,500,200,0]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,0]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,165]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,330]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,495]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,660]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,825]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,990]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,1155]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,300,1320]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,0]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,165]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,330]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,495]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,660]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,825]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,990]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,1155]
obj(670,170,165) placed at [T,X,Y,Z]=[5,0,470,1320]
obj(670,170,165) placed at [T,X,Y,Z]=[6,0,0,0]
obj(670,170,165) placed at [T,X,Y,Z]=[6,0,0,165]
obj(670,170,165) placed at [T,X,Y,Z]=[6,0,0,330]
obj(520,170,200) placed at [T,X,Y,Z]=[1,0,595,0]
obj(520,170,200) placed at [T,X,Y,Z]=[5,670,300,0]
obj(520,170,200) placed at [T,X,Y,Z]=[5,670,300,200]
obj(520,170,200) placed at [T,X,Y,Z]=[5,670,300,400]
obj(520,170,200) placed at [T,X,Y,Z]=[5,670,300,600]
obj(520,170,200) placed at [T,X,Y,Z]=[5,670,300,800]
obj(380,260,165) placed at [T,X,Y,Z]=[5,670,300,1000]
obj(380,260,165) placed at [T,X,Y,Z]=[5,670,300,1165]
obj(380,260,165) placed at [T,X,Y,Z]=[5,670,300,1330]
obj(380,260,165) placed at [T,X,Y,Z]=[5,670,470,0]
obj(583,547,50) placed at [T,X,Y,Z]=[1,520,0,422]
obj(583,547,50) placed at [T,X,Y,Z]=[4,0,0,1435]
obj(583,547,50) placed at [T,X,Y,Z]=[4,583,0,1435]
obj(583,547,50) placed at [T,X,Y,Z]=[6,0,0,495]
obj(583,547,50) placed at [T,X,Y,Z]=[6,0,0,545]
obj(583,547,50) placed at [T,X,Y,Z]=[6,0,0,595]
obj(583,547,50) placed at [T,X,Y,Z]=[6,0,0,645]
obj(583,547,50) placed at [T,X,Y,Z]=[6,0,0,695]
obj(583,547,50) placed at [T,X,Y,Z]=[6,0,0,745]
obj(583,547,50) placed at [T,X,Y,Z]=[6,0,0,795]
obj(377,260,157) placed at [T,X,Y,Z]=[5,670,470,165]
obj(377,260,157) placed at [T,X,Y,Z]=[5,670,470,322]
obj(377,260,157) placed at [T,X,Y,Z]=[5,670,470,479]
obj(377,260,157) placed at [T,X,Y,Z]=[5,670,470,636]
obj(377,260,157) placed at [T,X,Y,Z]=[5,670,470,793]
obj(377,260,157) placed at [T,X,Y,Z]=[6,0,0,845]
obj(720,397,50) placed at [T,X,Y,Z]=[3,440,0,200]
obj(610,140,140) placed at [T,X,Y,Z]=[0,0,395,1360]
obj(610,140,140) placed at [T,X,Y,Z]=[0,0,535,1360]
obj(610,140,140) placed at [T,X,Y,Z]=[3,440,200,0]
obj(275,260,150) placed at [T,X,Y,Z]=[1,880,0,472]
obj(275,260,150) placed at [T,X,Y,Z]=[1,880,0,622]
obj(275,260,150) placed at [T,X,Y,Z]=[1,880,0,772]
obj(275,260,150) placed at [T,X,Y,Z]=[1,880,0,922]
obj(275,260,150) placed at [T,X,Y,Z]=[1,880,0,1072]
obj(1200,90,90) placed at [T,X,Y,Z]=[0,0,690,0]
obj(1005,120,80) placed at [T,X,Y,Z]=[3,0,600,300]
obj(1200,155,50) placed at [T,X,Y,Z]=[3,0,400,1450]
obj(1200,155,50) placed at [T,X,Y,Z]=[3,0,555,1450]
obj(1200,155,50) placed at [T,X,Y,Z]=[4,0,547,1435]
obj(1200,155,50) placed at [T,X,Y,Z]=[5,0,640,950]
obj(1200,155,50) placed at [T,X,Y,Z]=[5,0,640,1000]
obj(1200,155,50) placed at [T,X,Y,Z]=[5,0,640,1050]
obj(1200,155,50) placed at [T,X,Y,Z]=[5,0,640,1100]
obj(1200,155,50) placed at [T,X,Y,Z]=[5,0,640,1150]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,150,1350]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,150,1400]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,150,1450]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,640,1200]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,640,1250]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,640,1300]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,640,1350]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,640,1400]
obj(1200,150,50) placed at [T,X,Y,Z]=[5,0,640,1450]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1002]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1052]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1102]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1152]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1202]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1252]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1302]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1352]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,0,1402]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1002]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1052]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1102]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1152]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1202]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1252]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1302]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1352]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,150,1402]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,0]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,50]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,100]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,150]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,200]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,250]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,300]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,350]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,170,400]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,260,845]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,260,895]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,260,945]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,995]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1045]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1095]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1145]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1195]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1245]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1295]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1345]
obj(1200,150,50) placed at [T,X,Y,Z]=[6,0,300,1395]
obj(1005,155,45) placed at [T,X,Y,Z]=[1,0,490,1455]
obj(1005,155,45) placed at [T,X,Y,Z]=[1,0,595,465]
obj(1005,155,45) placed at [T,X,Y,Z]=[1,0,645,1455]
obj(1005,155,45) placed at [T,X,Y,Z]=[6,0,0,1452]
obj(1005,155,45) placed at [T,X,Y,Z]=[6,0,155,1452]
obj(1005,155,45) placed at [T,X,Y,Z]=[6,0,170,450]
obj(250,250,100) placed at [T,X,Y,Z]=[0,670,0,1360]
obj(250,250,100) placed at [T,X,Y,Z]=[0,920,0,1360]
obj(300,155,130) placed at [T,X,Y,Z]=[1,880,260,472]
obj(300,155,130) placed at [T,X,Y,Z]=[1,880,260,602]
obj(300,155,130) placed at [T,X,Y,Z]=[1,880,260,732]
obj(300,155,130) placed at [T,X,Y,Z]=[1,880,260,862]
obj(275,190,100) placed at [T,X,Y,Z]=[1,880,260,992]
obj(275,190,100) placed at [T,X,Y,Z]=[1,880,260,1092]
obj(275,190,100) placed at [T,X,Y,Z]=[1,880,260,1192]
obj(300,170,100) placed at [T,X,Y,Z]=[1,880,415,472]
obj(300,170,100) placed at [T,X,Y,Z]=[1,880,415,572]
obj(300,170,100) placed at [T,X,Y,Z]=[1,880,415,672]
obj(300,170,100) placed at [T,X,Y,Z]=[1,880,415,772]
obj(300,170,100) placed at [T,X,Y,Z]=[1,880,415,872]
obj(300,170,100) placed at [T,X,Y,Z]=[1,880,450,972]
obj(300,170,100) placed at [T,X,Y,Z]=[1,880,450,1072]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,675,1360]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,90]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,190]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,290]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,390]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,490]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,590]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,690]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,790]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,890]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,990]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,1090]
obj(490,100,100) placed at [T,X,Y,Z]=[0,0,680,1190]
obj(210,150,155) placed at [T,X,Y,Z]=[1,960,0,0]
obj(210,150,155) placed at [T,X,Y,Z]=[1,960,150,0]
obj(1200,150,25) placed at [T,X,Y,Z]=[1,0,0,1462]
obj(1200,150,25) placed at [T,X,Y,Z]=[1,0,150,1462]
obj(1200,150,25) placed at [T,X,Y,Z]=[1,0,300,1462]
obj(1200,150,25) placed at [T,X,Y,Z]=[2,0,425,1475]
obj(1200,150,25) placed at [T,X,Y,Z]=[2,0,575,1475]
obj(1200,150,25) placed at [T,X,Y,Z]=[3,0,0,1465]
obj(270,145,110) placed at [T,X,Y,Z]=[1,880,450,1172]
obj(270,145,110) placed at [T,X,Y,Z]=[1,880,585,510]
obj(270,145,110) placed at [T,X,Y,Z]=[1,880,585,620]
obj(245,115,110) placed at [T,X,Y,Z]=[1,880,585,730]
obj(245,115,110) placed at [T,X,Y,Z]=[1,880,585,840]
obj(245,115,110) placed at [T,X,Y,Z]=[1,880,595,1172]
obj(245,110,110) placed at [T,X,Y,Z]=[0,490,690,90]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,200]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,510]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,560]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,610]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,660]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,710]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,760]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,810]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,860]
obj(860,45,50) placed at [T,X,Y,Z]=[1,0,700,910]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,700,960]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,700,1000]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,700,1040]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,700,1080]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,700,1120]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,710,1160]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,710,1200]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,710,1240]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,740,960]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,740,1000]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,740,1040]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,740,1080]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,740,1120]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,200]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,510]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,550]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,590]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,630]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,670]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,710]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,750]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,790]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,830]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,870]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,745,910]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,750,465]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,750,1160]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,750,1200]
obj(1200,40,40) placed at [T,X,Y,Z]=[1,0,750,1240]
obj(1200,40,40) placed at [T,X,Y,Z]=[2,0,700,0]
obj(370,85,60) placed at [T,X,Y,Z]=[0,670,250,1360]
obj(370,85,60) placed at [T,X,Y,Z]=[0,670,250,1420]
obj(370,85,60) placed at [T,X,Y,Z]=[0,0,680,1290]
obj(370,85,60) placed at [T,X,Y,Z]=[0,490,690,200]
obj(370,85,60) placed at [T,X,Y,Z]=[0,490,690,260]
obj(370,85,60) placed at [T,X,Y,Z]=[0,490,690,320]
obj(90,85,50) placed at [T,X,Y,Z]=[0,1040,250,1360]
obj(90,85,50) placed at [T,X,Y,Z]=[0,1040,250,1410]
obj(90,85,50) placed at [T,X,Y,Z]=[0,1020,340,0]
obj(90,85,50) placed at [T,X,Y,Z]=[0,1020,340,50]
obj(90,85,50) placed at [T,X,Y,Z]=[0,1020,340,100]
obj(720,20,20) placed at [T,X,Y,Z]=[0,0,675,1480]
obj(720,20,20) placed at [T,X,Y,Z]=[0,370,690,1290]
N = 7,
T1 = 2300,
T2 = 700 ? 
yes
% source_info

*/
