:- use_module(library(between)).
:- use_module(library(lists)).
:- use_module(library(clpfd)).
:- ensure_loaded(geosr).

% I suggest to run it on an example with 50 objects, 2 geometrical  
% dimension and a time dimension.
% In this example all time coordinates should be fixed and we should  
% just have 3 overlapping time intervals.

setup(Objects, Shapes, AllOIDs) :-
	findall(r(W,H), r(W,H), Rs),
	length(Rs, N),
	gen_geost(0, N, Rs, Objects, Shapes),
	numlist(1, N, AllOIDs).
% setup([object(1,1,[X1,Y1,0,100,100]),
%        object(2,2,[X2,Y2,50,100,150]),
%        object(3,3,[X3,Y3,110,100,210])],
%       [sbox(1,[0,0,0,0,0],[160,50,1,1,1]),
%        sbox(2,[0,0,0,0,0],[149,32,1,1,1]),
%        sbox(3,[0,0,0,0,0],[104,50,1,1,1])],
%       [1,2,3]) :-
% 	X1 in 0..40,
% 	Y1 in 0..100,
% 	X2 in 0..51,
% 	Y2 in 0..118,
% 	X3 in 0..96,
% 	Y3 in 0..100.

main(Objects) :-
	setup(Objects, Shapes, AllOIDs),
	geosr(Objects,
	      Shapes,
	      [% origin of a rectangle in dimension 1 or 2
	       (origin(O, S, D)	---> O^x(D)+S^t(D)),
	       % end of a rectangle in dimension 1 or 2
	       (end(O, S, D) ---> O^x(D)+S^t(D)+S^l(D)),
	       % start in time (use dimension 3)
	       (start(O) ---> O^x(3)),
	       % duration (use dimension 4)
	       (duration(O) ---> O^x(4)),
	       % end in time (use dimension 5)
	       (completion(O) ---> O^x(5)),
	       % pairwise overlap
	       (overlap(O, S, Oi, Si, D) --->
		   end(O, S, D) #> origin(Oi, Si, D) #/\
	           end(Oi, Si, D) #> origin(O, S, D)),
	       % Start+Duration=Completion
	       (start_dur_complete(OIDs) ---> 
		   forall(O, objects(OIDs),
			  start(O)+duration(O) #= completion(O))),
	       % non-overlapping between two objects considering
	       % both the geometrical and the time dimensions
	       (non_overlap(OIDs) ---> 
		   forall(O1, objects(OIDs),
			  forall(S1, sboxes([O1^sid]),
				 forall(O2, objects(OIDs),
					O1^oid #< O2^oid #=>
					forall(S2, sboxes([O2^sid]),
					       #\ (overlap(O1, S1, O2, S2, 1) #/\
						   overlap(O1, S1, O2, S2, 2) #/\
						   completion(O1) #> start(O2) #/\
						   completion(O2) #> start(O1))))))),
	       % all objects of OIDs are visible from dimension Dim and direction Dir
	       % if there do not exist any object O
	       % such that O is masked (by an other object) from dimension Dim and direction Dir
	       (visible(OIDs, Dim, Dir) ---> 
		   #\ exists(O, objects(OIDs), masked(OIDs, O, Dim, Dir))),
	       % an object O is masked according to dimension Dim and direction Dir
	       % if there exist an object Oi different from O that masks O according to Dim,Dir
	       (masked(OIDs, O, Dim, Dir) ---> 
		   exists(Oi, objects(OIDs), 
			  Oi^oid #\= O^oid #/\ masked_by(O, Oi, Dim, Dir))),
	       % an object O is masked by object Oi according to dimension Dim and direction Dir if ...
	       (masked_by(O, Oi, Dim, Dir) --->
		   exists(S, sboxes([O^sid]),
			  exists(Si, sboxes([Oi^sid]),
				 duration(O) #> 0 #/\
				 duration(Oi) #> 0 #/\
				 completion(O) #> start(Oi) #/\
				 completion(Oi) #> start(O) #/\
				 forall(D, [1,2], D #\= Dim #=> overlap(O, S, Oi, Si, D)) #/\
				 (Dir #= 0 #=> origin(O, S, Dim) #>= end(Oi, Si, Dim))  #/\
				 (Dir #= 1 #=> origin(Oi, Si, Dim) #>= end(O, S, Dim))  #/\
				 (start(O) #> start(Oi) #\/ completion(O) #< completion(Oi))))),
	       % BUSINESS RULES
	       start_dur_complete(AllOIDs),
	       non_overlap(AllOIDs),
	       visible(AllOIDs, 1, 0)]),
	order_objects(Objects, Ordered),
	label1(Ordered).

gen_geost(N, N, [], [], []) :- !.
gen_geost(I, N, [r(W,H)|Rs], [Object|Objects], [Shape|Shapes]) :-
	J is I+1,
	Object = object(J,J,[X,Y,S,D,C]),
	Shape = sbox(J,[0,0,0,0,0],[W,H,1,1,1]),
	LimX is 200-W,
	LimY is 400-H,
	X in 0..LimX,
	Y in 0..LimY,
	(3*I < N -> S=0 ; 3*I < 2*N -> S=50 ; S=110),
	D = 100,
	C is S+100,
	gen_geost(J, N, Rs, Objects, Shapes).

order_objects(Objects, Ordered) :-
	length(Part1, 16),
	length(Part2, 16),
	length(Part3, 16),
	append(Part2, Part3, Part23),
	append(Part1, Part23, Objects),
	order_objects(Part1, Part2, Ordered, Part3).

order_objects([], []) --> [].
order_objects([X|Xs], [Y|Ys]) --> [X,Y],
	order_objects(Xs, Ys).


label1([]).
label1([O|Objects]) :-
	O = object(_,_,[X,Y|_]),
	labeling([up], [X,Y]),
	% portray_clause(O),
	label2(Objects).

label2([]).
label2([O|Objects]) :-
	O = object(_,_,[X,Y|_]),
	labeling([up], [X]),
	labeling([down], [Y]),
	% portray_clause(O),
	label1(Objects).

% rectangles: start/end = 1/100, 50/150, or 110/200

r(160,50).
r(149,32).
r(104,50).
r(98,30).
r(82,34).
r(82,20).
r(70,32).
r(70,26).
r(68,32).
r(68,8).
r(64,16).
r(63,24).
r(54,12).
r(53,16).
r(53,9).
r(52,16).
r(51,46).
r(46,34).
r(46,16).
r(45,16).
r(44,8).
r(42,30).
r(42,12).
r(42,10).
r(40,17).
r(40,16).
r(40,10).
r(36,12).
r(32,14).
r(25,20).
r(25,20).
r(24,20).
r(24,12).
r(24,12).
r(20,52).
r(16,22).
r(16,19).
r(16,19).
r(16,17).
r(14,50).
r(12,32).
r(12,32).
r(12,24).
r(12,14).
r(10,21).
r(6,22).
r(4,21).
r(4,12).

end_of_file.

| ?- main(O).
object(1, 1, [0,0,0,100,100]).
object(17, 17, [0,354,50,100,150]).
object(2, 2, [0,50,0,100,100]).
object(18, 18, [0,320,50,100,150]).
object(3, 3, [0,82,0,100,100]).
object(19, 19, [0,304,50,100,150]).
object(4, 4, [0,132,0,100,100]).
object(20, 20, [0,288,50,100,150]).
object(5, 5, [0,162,0,100,100]).
object(21, 21, [0,280,50,100,150]).
object(6, 6, [0,196,0,100,100]).
object(22, 22, [0,250,50,100,150]).
object(7, 7, [0,216,0,100,100]).
object(23, 23, [42,268,50,100,150]).
object(8, 8, [70,216,0,100,100]).
object(24, 24, [42,258,50,100,150]).
object(9, 9, [82,162,0,100,100]).
object(25, 25, [45,287,50,100,150]).
object(10, 10, [70,242,0,100,100]).
object(26, 26, [46,338,50,100,150]).
object(11, 11, [82,194,0,100,100]).
object(27, 27, [46,328,50,100,150]).
object(12, 12, [98,132,0,100,100]).
object(28, 28, [46,316,50,100,150]).
object(13, 13, [104,82,0,100,100]).
object(29, 29, [51,386,50,100,150]).
object(14, 14, [104,94,0,100,100]).
object(30, 30, [51,366,50,100,150]).
object(15, 15, [104,110,0,100,100]).
object(31, 31, [76,366,50,100,150]).
object(16, 16, [140,210,0,100,100]).
object(32, 32, [82,308,50,100,150]).
object(33, 33, [0,0,110,100,210]).
object(34, 34, [0,238,110,100,210]).
object(35, 35, [0,12,110,100,210]).
object(36, 36, [0,216,110,100,210]).
object(37, 37, [0,64,110,100,210]).
object(38, 38, [0,197,110,100,210]).
object(39, 39, [0,83,110,100,210]).
object(40, 40, [0,147,110,100,210]).
object(41, 41, [0,100,110,100,210]).
object(42, 42, [12,115,110,100,210]).
object(43, 43, [14,147,110,100,210]).
object(44, 44, [0,133,110,100,210]).
object(45, 45, [14,171,110,100,210]).
object(46, 46, [16,216,110,100,210]).
object(47, 47, [16,64,110,100,210]).
object(48, 48, [12,103,110,100,210]).
